@@ -7,7 +7,6 @@
 #include "mozilla/dom/cache/ReadStream.h"
 
 #include "mozilla/SnappyUncompressInputStream.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/cache/CacheStreamControlChild.h"
 #include "mozilla/dom/cache/CacheStreamControlParent.h"
 #include "mozilla/dom/cache/CacheTypes.h"
@@ -19,8 +18,6 @@
 
 namespace mozilla::dom::cache {
 
-using mozilla::Unused;
-
 // ----------------------------------------------------------------------------
 
 // The inner stream class.  This is where all of the real work is done.  As
@@ -498,7 +495,7 @@ void ReadStream::Inner::OpenStreamFailed() {
   MOZ_DIAGNOSTIC_ASSERT(!mStream);
   MOZ_DIAGNOSTIC_ASSERT(!mSnappyStream);
   mMutex.AssertCurrentThreadOwns();
-  Unused << NS_NewCStringInputStream(getter_AddRefs(mStream), ""_ns);
+  (void)NS_NewCStringInputStream(getter_AddRefs(mStream), ""_ns);
   mSnappyStream = mStream;
   mStream->Close();
   NoteClosed();
