@@ -2,35 +2,34 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/dom/FetchService.h"
+
 #include "FetchLog.h"
 #include "FetchParent.h"
+#include "mozilla/BasePrincipal.h"
+#include "mozilla/ClearOnShutdown.h"
+#include "mozilla/SchedulerGroup.h"
+#include "mozilla/UniquePtr.h"
+#include "mozilla/dom/ClientInfo.h"
+#include "mozilla/dom/InternalRequest.h"
+#include "mozilla/dom/InternalResponse.h"
+#include "mozilla/dom/PerformanceStorage.h"
+#include "mozilla/dom/PerformanceTiming.h"
+#include "mozilla/dom/ServiceWorkerDescriptor.h"
+#include "mozilla/ipc/BackgroundUtils.h"
+#include "mozilla/net/CookieJarSettings.h"
 #include "nsContentUtils.h"
 #include "nsIContentSecurityPolicy.h"
 #include "nsICookieJarSettings.h"
+#include "nsIIOService.h"
 #include "nsILoadGroup.h"
 #include "nsILoadInfo.h"
-#include "nsIIOService.h"
 #include "nsIObserverService.h"
 #include "nsIPrincipal.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsNetUtil.h"
 #include "nsThreadUtils.h"
 #include "nsXULAppAPI.h"
-#include "mozilla/BasePrincipal.h"
-#include "mozilla/ClearOnShutdown.h"
-#include "mozilla/SchedulerGroup.h"
-#include "mozilla/ScopeExit.h"
-#include "mozilla/UniquePtr.h"
-#include "mozilla/dom/ClientInfo.h"
-#include "mozilla/dom/FetchService.h"
-#include "mozilla/dom/InternalRequest.h"
-#include "mozilla/dom/InternalResponse.h"
-#include "mozilla/dom/PerformanceStorage.h"
-#include "mozilla/dom/PerformanceTiming.h"
-#include "mozilla/dom/ServiceWorkerDescriptor.h"
-#include "mozilla/glean/NetwerkMetrics.h"
-#include "mozilla/ipc/BackgroundUtils.h"
-#include "mozilla/net/CookieJarSettings.h"
 
 namespace mozilla::dom {
 
@@ -45,9 +44,9 @@ FetchServicePromises::FetchServicePromises()
           MakeRefPtr<FetchServiceResponseTimingPromise::Private>(__func__)),
       mEndPromise(
           MakeRefPtr<FetchServiceResponseEndPromise::Private>(__func__)) {
-  mAvailablePromise->UseSynchronousTaskDispatch(__func__);
-  mTimingPromise->UseSynchronousTaskDispatch(__func__);
-  mEndPromise->UseSynchronousTaskDispatch(__func__);
+  mAvailablePromise->UseDirectTaskDispatch(__func__);
+  mTimingPromise->UseDirectTaskDispatch(__func__);
+  mEndPromise->UseDirectTaskDispatch(__func__);
 }
 
 RefPtr<FetchServiceResponseAvailablePromise>
@@ -265,7 +264,7 @@ RefPtr<FetchServicePromises> FetchService::FetchInstance::Fetch() {
       // tracking fetch to false, but for Keepalive
       // requests from main thread this needs to be
       // changed. See Bug 1892406
-      false  // IsTrackingFetch
+      net::ClassificationFlags({0, 0})  // TrackingFlags
   );
 
   if (mArgsType == FetchArgsType::WorkerFetch) {
@@ -789,21 +788,11 @@ bool FetchService::DoesExceedsKeepaliveResourceLimits(
     const nsACString& origin) {
   if (mTotalKeepAliveRequests >=
       StaticPrefs::dom_fetchKeepalive_total_request_limit()) {
-    // Count keep-alive request discards due to
-    // exceeding the total keep-alive request limit.
-    mozilla::glean::networking::fetch_keepalive_discard_count
-        .Get("total_keepalive_limit"_ns)
-        .Add(1);
     return true;
   }
 
   if (mPendingKeepAliveRequestsPerOrigin.Get(origin) >=
       StaticPrefs::dom_fetchKeepalive_request_limit_per_origin()) {
-    // Count keep-alive request discards due to
-    // exceeding the per-origin request limit.
-    mozilla::glean::networking::fetch_keepalive_discard_count
-        .Get("per_origin_limit"_ns)
-        .Add(1);
     return true;
   }
 
