@@ -3,25 +3,34 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "FFmpegLibWrapper.h"
+
 #include "FFmpegLog.h"
 #include "mozilla/PodOperations.h"
 #ifdef MOZ_FFMPEG
 #  include "mozilla/StaticPrefs_media.h"
 #endif
-#include "mozilla/Types.h"
 #include "PlatformDecoderModule.h"
 #include "prlink.h"
 #ifdef MOZ_WIDGET_GTK
-#  include "mozilla/gfx/gfxVars.h"
-#  include "mozilla/widget/DMABufLibWrapper.h"
 #  include "VALibWrapper.h"
+#  include "mozilla/gfx/gfxVars.h"
+#  include "mozilla/widget/DMABufDevice.h"
 #endif
 
-#define AV_LOG_INFO 32
+#define AV_LOG_QUIET -8
+#define AV_LOG_PANIC 0
+#define AV_LOG_FATAL 8
+#define AV_LOG_ERROR 16
 #define AV_LOG_WARNING 24
+#define AV_LOG_INFO 32
+#define AV_LOG_VERBOSE 40
+#define AV_LOG_DEBUG 48
+#define AV_LOG_TRACE 56
 
 namespace mozilla {
 
+static LazyLogModule sFFmpegLibLog("FFmpegLib");
+
 FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
   if (!mAVCodecLib || !mAVUtilLib) {
     Unlink();
@@ -71,6 +80,7 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
     AV_FUNC_59 = 1 << 6,
     AV_FUNC_60 = 1 << 7,
     AV_FUNC_61 = 1 << 8,
+    AV_FUNC_62 = 1 << 9,
     AV_FUNC_AVUTIL_53 = AV_FUNC_53 | AV_FUNC_AVUTIL_MASK,
     AV_FUNC_AVUTIL_54 = AV_FUNC_54 | AV_FUNC_AVUTIL_MASK,
     AV_FUNC_AVUTIL_55 = AV_FUNC_55 | AV_FUNC_AVUTIL_MASK,
@@ -80,9 +90,10 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
     AV_FUNC_AVUTIL_59 = AV_FUNC_59 | AV_FUNC_AVUTIL_MASK,
     AV_FUNC_AVUTIL_60 = AV_FUNC_60 | AV_FUNC_AVUTIL_MASK,
     AV_FUNC_AVUTIL_61 = AV_FUNC_61 | AV_FUNC_AVUTIL_MASK,
+    AV_FUNC_AVUTIL_62 = AV_FUNC_62 | AV_FUNC_AVUTIL_MASK,
     AV_FUNC_AVCODEC_ALL = AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 | AV_FUNC_56 |
                           AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
-                          AV_FUNC_61,
+                          AV_FUNC_61 | AV_FUNC_62,
     AV_FUNC_AVUTIL_ALL = AV_FUNC_AVCODEC_ALL | AV_FUNC_AVUTIL_MASK
   };
 
@@ -114,6 +125,9 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
     case 61:
       version = AV_FUNC_61;
       break;
+    case 62:
+      version = AV_FUNC_62;
+      break;
     default:
       FFMPEGV_LOG("Unknown avcodec version: %d", macro);
       Unlink();
@@ -154,17 +168,20 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
   AV_FUNC(av_lockmgr_register, AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 |
                                    AV_FUNC_56 | AV_FUNC_57 | AV_FUNC_58)
   AV_FUNC(avcodec_alloc_context3, AV_FUNC_AVCODEC_ALL)
-  AV_FUNC(avcodec_close, AV_FUNC_AVCODEC_ALL)
+  AV_FUNC(avcodec_close, AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 | AV_FUNC_56 |
+                             AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
+                             AV_FUNC_61)
+
   AV_FUNC(avcodec_decode_audio4, AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 |
                                      AV_FUNC_56 | AV_FUNC_57 | AV_FUNC_58)
   AV_FUNC(avcodec_decode_video2, AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 |
                                      AV_FUNC_56 | AV_FUNC_57 | AV_FUNC_58)
   AV_FUNC(avcodec_find_decoder, AV_FUNC_AVCODEC_ALL)
   AV_FUNC(avcodec_find_decoder_by_name,
-          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC(avcodec_find_encoder, AV_FUNC_AVCODEC_ALL)
   AV_FUNC(avcodec_find_encoder_by_name,
-          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC(avcodec_flush_buffers, AV_FUNC_AVCODEC_ALL)
   AV_FUNC(avcodec_open2, AV_FUNC_AVCODEC_ALL)
   AV_FUNC(avcodec_register_all, AV_FUNC_53 | AV_FUNC_54 | AV_FUNC_55 |
@@ -179,59 +196,67 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
   AV_FUNC(avcodec_get_frame_defaults, (AV_FUNC_53 | AV_FUNC_54))
   AV_FUNC(avcodec_free_frame, AV_FUNC_54)
   AV_FUNC(avcodec_send_packet,
-          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC(avcodec_receive_packet,
-          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC(avcodec_send_frame, AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC(avcodec_send_frame,
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC(avcodec_receive_frame,
-          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+          AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC(avcodec_default_get_buffer2,
           (AV_FUNC_55 | AV_FUNC_56 | AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 |
-           AV_FUNC_60 | AV_FUNC_61))
-  AV_FUNC(av_packet_alloc,
-          (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61))
-  AV_FUNC(av_packet_unref,
-          (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61))
-  AV_FUNC(av_packet_free,
-          (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61))
+           AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62))
+  AV_FUNC(av_packet_alloc, (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
+                            AV_FUNC_61 | AV_FUNC_62))
+  AV_FUNC(av_packet_unref, (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
+                            AV_FUNC_61 | AV_FUNC_62))
+  AV_FUNC(av_packet_free, (AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
+                           AV_FUNC_61 | AV_FUNC_62))
   AV_FUNC(avcodec_descriptor_get, AV_FUNC_AVCODEC_ALL)
+  AV_FUNC(av_log_set_callback, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_log_set_level, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_malloc, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_freep, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_frame_alloc,
           (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
            AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
-           AV_FUNC_AVUTIL_61))
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
+  AV_FUNC(av_frame_clone,
+          (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
+           AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
   AV_FUNC(av_frame_free,
           (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
            AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
-           AV_FUNC_AVUTIL_61))
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
   AV_FUNC(av_frame_unref,
           (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
            AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
-           AV_FUNC_AVUTIL_61))
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
   AV_FUNC(av_frame_get_buffer,
           (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
            AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
-           AV_FUNC_AVUTIL_61))
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
   AV_FUNC(av_frame_make_writable,
           (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
            AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
-           AV_FUNC_AVUTIL_61))
+           AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62))
   AV_FUNC(av_image_check_size, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_image_get_buffer_size, AV_FUNC_AVUTIL_ALL)
   AV_FUNC_OPTION(av_channel_layout_default,
-                 AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                 AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION(av_channel_layout_from_mask,
-                 AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
-  AV_FUNC_OPTION(av_channel_layout_copy, AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                 AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62)
+  AV_FUNC_OPTION(av_channel_layout_copy,
+                 AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION(av_buffer_get_opaque,
                  (AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 | AV_FUNC_AVUTIL_58 |
-                  AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61))
-  AV_FUNC(
-      av_buffer_create,
-      (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
-       AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 | AV_FUNC_61))
+                  AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                  AV_FUNC_AVUTIL_62))
+  AV_FUNC(av_buffer_create,
+          (AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
+           AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
+           AV_FUNC_61 | AV_FUNC_62))
   AV_FUNC_OPTION(av_frame_get_colorspace,
                  AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
                      AV_FUNC_AVUTIL_58)
@@ -239,79 +264,93 @@ FFmpegLibWrapper::LinkResult FFmpegLibWrapper::Link() {
                  AV_FUNC_AVUTIL_55 | AV_FUNC_AVUTIL_56 | AV_FUNC_AVUTIL_57 |
                      AV_FUNC_AVUTIL_58)
   AV_FUNC(av_strerror, AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                           AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                           AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                           AV_FUNC_AVUTIL_62)
   AV_FUNC(av_get_sample_fmt_name, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_dict_set, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_dict_free, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_opt_set, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_opt_set_double, AV_FUNC_AVUTIL_ALL)
   AV_FUNC(av_opt_set_int, AV_FUNC_AVUTIL_ALL)
-  AV_FUNC(avcodec_free_context,
-          AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(avcodec_get_hw_config,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_codec_is_decoder,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_codec_iterate,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+  AV_FUNC(avcodec_free_context, AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 |
+                                    AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(avcodec_get_hw_config, AV_FUNC_58 | AV_FUNC_59 |
+                                                   AV_FUNC_60 | AV_FUNC_61 |
+                                                   AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(av_codec_is_decoder, AV_FUNC_58 | AV_FUNC_59 |
+                                                 AV_FUNC_60 | AV_FUNC_61 |
+                                                 AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(av_codec_is_encoder, AV_FUNC_58 | AV_FUNC_59 |
+                                                 AV_FUNC_60 | AV_FUNC_61 |
+                                                 AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(av_codec_iterate, AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 |
+                                              AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC_OPTION_SILENT(av_hwdevice_ctx_init,
                         AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                            AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION_SILENT(av_hwdevice_ctx_alloc,
                         AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
-  AV_FUNC_OPTION_SILENT(av_buffer_ref, AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                                           AV_FUNC_AVUTIL_60 |
-                                           AV_FUNC_AVUTIL_61)
+                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                            AV_FUNC_AVUTIL_62)
+  AV_FUNC_OPTION_SILENT(
+      av_buffer_ref, AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 | AV_FUNC_AVUTIL_60 |
+                         AV_FUNC_AVUTIL_61 | AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION_SILENT(av_buffer_unref, AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
                                              AV_FUNC_AVUTIL_60 |
-                                             AV_FUNC_AVUTIL_61)
+                                             AV_FUNC_AVUTIL_61 |
+                                             AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION_SILENT(av_hwframe_ctx_alloc,
                         AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                            AV_FUNC_AVUTIL_62)
   AV_FUNC_OPTION_SILENT(av_hwframe_ctx_init,
                         AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                            AV_FUNC_AVUTIL_62)
 
 #ifdef MOZ_WIDGET_GTK
-  AV_FUNC_OPTION_SILENT(av_hwdevice_hwconfig_alloc,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_hwdevice_get_hwframe_constraints,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_hwframe_constraints_free,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_hwframe_transfer_get_formats,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
-  AV_FUNC_OPTION_SILENT(av_hwdevice_ctx_create_derived,
-                        AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61)
+  AV_FUNC_OPTION_SILENT(av_hwdevice_hwconfig_alloc, AV_FUNC_58 | AV_FUNC_59 |
+                                                        AV_FUNC_60 |
+                                                        AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(
+      av_hwdevice_get_hwframe_constraints,
+      AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(
+      av_hwframe_constraints_free,
+      AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(
+      av_hwframe_transfer_get_formats,
+      AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
+  AV_FUNC_OPTION_SILENT(
+      av_hwdevice_ctx_create_derived,
+      AV_FUNC_58 | AV_FUNC_59 | AV_FUNC_60 | AV_FUNC_61 | AV_FUNC_62)
   AV_FUNC_OPTION_SILENT(avcodec_get_name, AV_FUNC_57 | AV_FUNC_58 | AV_FUNC_59 |
-                                              AV_FUNC_60 | AV_FUNC_61)
+                                              AV_FUNC_60 | AV_FUNC_61 |
+                                              AV_FUNC_62)
   AV_FUNC_OPTION_SILENT(av_get_pix_fmt_string,
                         AV_FUNC_AVUTIL_58 | AV_FUNC_AVUTIL_59 |
-                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61)
+                            AV_FUNC_AVUTIL_60 | AV_FUNC_AVUTIL_61 |
+                            AV_FUNC_AVUTIL_62)
 #endif
 
   AV_FUNC_OPTION(av_tx_init, AV_FUNC_AVUTIL_ALL)
   AV_FUNC_OPTION(av_tx_uninit, AV_FUNC_AVUTIL_ALL)
 
+#ifdef MOZ_WIDGET_ANDROID
+  AV_FUNC(av_mediacodec_release_buffer, AV_FUNC_AVCODEC_ALL);
+  AV_FUNC(moz_avcodec_mediacodec_is_eos, AV_FUNC_AVCODEC_ALL);
+#endif
+
 #undef AV_FUNC
 #undef AV_FUNC_OPTION
 
   if (avcodec_register_all) {
     avcodec_register_all();
   }
-  int logLevel = 0;
-  const char* ffmpegLogLevel = getenv("MOZ_AV_LOG_LEVEL");
-  if (ffmpegLogLevel && *ffmpegLogLevel) {
-    logLevel = atoi(ffmpegLogLevel);
-  } else if (MOZ_LOG_TEST(sFFmpegVideoLog, LogLevel::Debug) ||
-             MOZ_LOG_TEST(sFFmpegAudioLog, LogLevel::Debug)) {
-    logLevel = AV_LOG_WARNING;
-  } else if (MOZ_LOG_TEST(sFFmpegVideoLog, LogLevel::Info) ||
-             MOZ_LOG_TEST(sFFmpegAudioLog, LogLevel::Info)) {
-    logLevel = AV_LOG_INFO;
-  }
-  av_log_set_level(logLevel);
+
+  UpdateLogLevel();
+  av_log_set_callback(Log);
   return LinkResult::Success;
 }
 
@@ -336,6 +375,77 @@ void FFmpegLibWrapper::Unlink() {
   PodZero(this);
 }
 
+void FFmpegLibWrapper::UpdateLogLevel() {
+  LogModule* mod = sFFmpegLibLog;
+  av_log_set_level(ToLibLogLevel(mod->Level()));
+}
+
+/* static */ void FFmpegLibWrapper::RegisterCallbackLogLevel(
+    PrefChangedFunc aCallback) {
+  if (!NS_IsMainThread()) {
+    NS_DispatchToMainThread(NS_NewRunnableFunction(
+        __func__, [aCallback]() { RegisterCallbackLogLevel(aCallback); }));
+    return;
+  }
+
+  Preferences::RegisterCallback(aCallback, "logging.FFmpegLib"_ns);
+}
+
+/* static */ int FFmpegLibWrapper::ToLibLogLevel(LogLevel aLevel) {
+  switch (aLevel) {
+    case LogLevel::Disabled:
+      return AV_LOG_QUIET;
+    case LogLevel::Error:
+      return AV_LOG_ERROR;
+    case LogLevel::Warning:
+      return AV_LOG_WARNING;
+    case LogLevel::Info:
+      return AV_LOG_INFO;
+    case LogLevel::Debug:
+      return AV_LOG_DEBUG;
+    case LogLevel::Verbose:
+      return AV_LOG_TRACE;
+    default:
+      MOZ_ASSERT_UNREACHABLE("Unhandled log level!");
+      break;
+  }
+  return AV_LOG_QUIET;
+}
+
+/* static */ LogLevel FFmpegLibWrapper::FromLibLogLevel(int aLevel) {
+  switch (aLevel) {
+    case AV_LOG_QUIET:
+      return LogLevel::Disabled;
+    case AV_LOG_PANIC:
+    case AV_LOG_FATAL:
+    case AV_LOG_ERROR:
+      return LogLevel::Error;
+    case AV_LOG_WARNING:
+      return LogLevel::Warning;
+    case AV_LOG_INFO:
+      return LogLevel::Info;
+    case AV_LOG_DEBUG:
+      return LogLevel::Debug;
+    case AV_LOG_VERBOSE:
+    case AV_LOG_TRACE:
+      return LogLevel::Verbose;
+    default:
+      MOZ_ASSERT_UNREACHABLE("Unhandled log level!");
+      break;
+  }
+  return LogLevel::Disabled;
+}
+
+/* static */ void FFmpegLibWrapper::Log(void* aPtr, int aLevel,
+                                        const char* aFmt, va_list aArgs) {
+  LogLevel level = FromLibLogLevel(aLevel);
+  if (MOZ_LOG_TEST(sFFmpegLibLog, level)) {
+    nsAutoCString msg;
+    msg.AppendVprintf(aFmt, aArgs);
+    MOZ_LOG(sFFmpegLibLog, level, ("[%p] %s", aPtr, msg.get()));
+  }
+}
+
 #ifdef MOZ_WIDGET_GTK
 bool FFmpegLibWrapper::IsVAAPIAvailable() {
 #  define VA_FUNC_LOADED(func) ((func) != nullptr)
