@@ -21,7 +21,6 @@
 #include "mozilla/ErrorResult.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/StaticPtr.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/PWebrtcGlobal.h"
 #include "mozilla/dom/PWebrtcGlobalChild.h"
@@ -194,7 +193,7 @@ void WebrtcGlobalInformation::ClearAllStats(const GlobalObject& aGlobal) {
   if (!WebrtcContentParents::Empty()) {
     // Pass on the request to any content process based PeerConnections.
     for (const auto& cp : WebrtcContentParents::GetAll()) {
-      Unused << cp->SendClearStats();
+      (void)cp->SendClearStats();
     }
   }
 
@@ -447,12 +446,12 @@ void WebrtcGlobalInformation::ClearLogging(const GlobalObject& aGlobal) {
   if (!WebrtcContentParents::Empty()) {
     // Clear content process signaling logs
     for (const auto& cp : WebrtcContentParents::GetAll()) {
-      Unused << cp->SendClearLog();
+      (void)cp->SendClearLog();
     }
   }
 
   // Clear chrome process signaling logs
-  Unused << RunLogClear();
+  (void)RunLogClear();
 }
 
 static RefPtr<GenericPromise> UpdateLogStash() {
@@ -548,7 +547,7 @@ void WebrtcGlobalInformation::SetAecDebug(const GlobalObject& aGlobal,
   sLastAECDebug = aEnable;
 
   for (const auto& cp : WebrtcContentParents::GetAll()) {
-    Unused << cp->SendSetAecLogging(aEnable);
+    (void)cp->SendSetAecLogging(aEnable);
   }
 }
 
@@ -605,7 +604,7 @@ void WebrtcGlobalInformation::AdjustTimerReferences(
             },
             nullptr, WebrtcGlobalStatsHistory::Pref::PollIntervalMs(),
             nsITimer::TYPE_REPEATING_SLACK,
-            "WebrtcGlobalInformation::GatherHistory");
+            "WebrtcGlobalInformation::GatherHistory"_ns);
       }
       ClearOnShutdown(&sHistoryTimer);
     }
