@@ -25,13 +25,11 @@
 #include "mozilla/Preferences.h"
 #include "mozilla/RemoteLazyInputStreamStorage.h"
 #include "mozilla/Result.h"
-#include "mozilla/ResultExtensions.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/Services.h"
 #include "mozilla/StaticPrefs_dom.h"
 #include "mozilla/StaticPrefs_privacy.h"
 #include "mozilla/StoragePrincipalHelper.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/Client.h"
 #include "mozilla/dom/ClientIPCTypes.h"
 #include "mozilla/dom/ClientManager.h"
@@ -354,7 +352,7 @@ Result<IPCInternalRequest, nsresult> GetIPCInternalRequest(
   nsCOMPtr<nsIReferrerInfo> referrerInfo = httpChannel->GetReferrerInfo();
   if (referrerInfo) {
     referrerPolicy = referrerInfo->ReferrerPolicy();
-    Unused << referrerInfo->GetComputedReferrerSpec(referrer);
+    (void)referrerInfo->GetComputedReferrerSpec(referrer);
   }
 
   uint32_t loadFlags;
@@ -1105,8 +1103,8 @@ nsresult ServiceWorkerPrivate::SendFetchEvent(
     nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
 
     // We'll check for a null registration below rather than an error code here.
-    Unused << swm->GetClientRegistration(loadInfo->GetClientInfo().ref(),
-                                         getter_AddRefs(registration));
+    (void)swm->GetClientRegistration(loadInfo->GetClientInfo().ref(),
+                                     getter_AddRefs(registration));
   }
 
   // Its possible the registration is removed between starting the interception
@@ -1144,8 +1142,7 @@ nsresult ServiceWorkerPrivate::SendFetchEvent(
     Shutdown();
   });
 
-  IPCInternalRequest request;
-  MOZ_TRY_VAR(request, GetIPCInternalRequest(aChannel));
+  IPCInternalRequest request = MOZ_TRY(GetIPCInternalRequest(aChannel));
 
   scopeExit.release();
 
@@ -1214,7 +1211,7 @@ nsresult ServiceWorkerPrivate::SendFetchEventInternal(
       ->Then(GetCurrentSerialEventTarget(), __func__,
              [holder = std::move(holder)](
                  const GenericPromise::ResolveOrRejectValue& aResult) {
-               Unused << NS_WARN_IF(aResult.IsReject());
+               (void)NS_WARN_IF(aResult.IsReject());
              });
 
   return NS_OK;
@@ -1395,7 +1392,7 @@ void ServiceWorkerPrivate::UpdateState(ServiceWorkerState aState) {
   }
 
   for (auto& event : mPendingFunctionalEvents) {
-    Unused << NS_WARN_IF(NS_FAILED(event->Send()));
+    (void)NS_WARN_IF(NS_FAILED(event->Send()));
   }
 
   mPendingFunctionalEvents.Clear();
@@ -1961,7 +1958,7 @@ RefPtr<GenericNonExclusivePromise> ServiceWorkerPrivate::ShutdownInternal(
   RefPtr<GenericNonExclusivePromise::Private> promise =
       new GenericNonExclusivePromise::Private(__func__);
 
-  Unused << ExecServiceWorkerOp(
+  (void)ExecServiceWorkerOp(
       ServiceWorkerTerminateWorkerOpArgs(aShutdownStateId),
       // It doesn't make sense to extend the lifetime in this case.  This will
       // also ensure that we don't try and spawn the ServiceWorker, but as our
