@@ -4,16 +4,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/dom/PublicKeyCredential.h"
+
 #include "mozilla/Base64.h"
 #include "mozilla/HoldDropJSObjects.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/StaticPrefs_security.h"
 #include "mozilla/dom/AuthenticatorResponse.h"
-#include "mozilla/dom/CredentialsContainer.h"
 #include "mozilla/dom/ChromeUtils.h"
+#include "mozilla/dom/CredentialsContainer.h"
 #include "mozilla/dom/Navigator.h"
 #include "mozilla/dom/Promise.h"
-#include "mozilla/dom/PublicKeyCredential.h"
 #include "mozilla/dom/WebAuthenticationBinding.h"
 #include "mozilla/dom/WebAuthnHandler.h"
 #include "nsCycleCollectionParticipant.h"
@@ -178,15 +179,21 @@ already_AddRefed<Promise> PublicKeyCredential::GetClientCapabilities(
   entry->mKey = u"extension:credProps"_ns;
   entry->mValue = true;
 
-  // Bug 1844449: support the credProtect extension.
-  // entry = capabilities.Entries().AppendElement();
-  // entry->mKey = u"extension:credentialProtectionPolicy"_ns;
-  // entry->mValue = true;
+  entry = capabilities.Entries().AppendElement();
+  entry->mKey = u"extension:credentialProtectionPolicy"_ns;
+#if defined(XP_MACOSX) || defined(MOZ_WIDGET_ANDROID)
+  entry->mValue = false;
+#else
+  entry->mValue = true;
+#endif
 
-  // Bug 1844449: support the credProtect extension.
-  // entry = capabilities.Entries().AppendElement();
-  // entry->mKey = u"extension:enforceCredentialProtectionPolicy"_ns;
-  // entry->mValue = true;
+  entry = capabilities.Entries().AppendElement();
+  entry->mKey = u"extension:enforceCredentialProtectionPolicy"_ns;
+#if defined(XP_MACOSX) || defined(MOZ_WIDGET_ANDROID)
+  entry->mValue = false;
+#else
+  entry->mValue = true;
+#endif
 
   // Bug 1844448: support the credBlob extension.
   // entry = capabilities.Entries().AppendElement();
@@ -197,14 +204,21 @@ already_AddRefed<Promise> PublicKeyCredential::GetClientCapabilities(
   entry->mKey = u"extension:hmacCreateSecret"_ns;
   entry->mValue = true;
 
+  entry = capabilities.Entries().AppendElement();
+  entry->mKey = u"extension:largeBlob"_ns;
+#if defined(XP_MACOSX) || defined(XP_WIN)
+  entry->mValue = true;
+#else
+  entry->mValue = false;
+#endif
+
   entry = capabilities.Entries().AppendElement();
   entry->mKey = u"extension:minPinLength"_ns;
   entry->mValue = true;
 
-  // Bug 1863819: support the PRF extension
-  // entry = capabilities.Entries().AppendElement();
-  // entry->mKey = u"extension:prf"_ns;
-  // entry->mValue = true;
+  entry = capabilities.Entries().AppendElement();
+  entry->mKey = u"extension:prf"_ns;
+  entry->mValue = true;
 
   entry = capabilities.Entries().AppendElement();
   entry->mKey = u"hybridTransport"_ns;
@@ -282,6 +296,26 @@ void PublicKeyCredential::GetClientExtensionResults(
         mClientExtensionOutputs.mHmacCreateSecret.Value());
   }
 
+  if (mClientExtensionOutputs.mLargeBlob.WasPassed()) {
+    const AuthenticationExtensionsLargeBlobOutputs& src =
+        mClientExtensionOutputs.mLargeBlob.Value();
+    AuthenticationExtensionsLargeBlobOutputs& dest =
+        aResult.mLargeBlob.Construct();
+
+    if (src.mSupported.WasPassed()) {
+      dest.mSupported.Construct(src.mSupported.Value());
+    }
+
+    if (src.mWritten.WasPassed()) {
+      dest.mWritten.Construct(src.mWritten.Value());
+    }
+
+    if (mLargeBlobValue.isSome()) {
+      dest.mBlob.Construct().Init(
+          TypedArrayCreator<ArrayBuffer>(mLargeBlobValue.ref()).Create(cx));
+    }
+  }
+
   if (mClientExtensionOutputs.mPrf.WasPassed()) {
     AuthenticationExtensionsPRFOutputs& dest = aResult.mPrf.Construct();
 
@@ -336,6 +370,15 @@ void PublicKeyCredential::ToJSON(JSContext* aCx,
             mClientExtensionOutputs.mPrf.Value().mEnabled.Value());
       }
     }
+    if (mClientExtensionOutputs.mLargeBlob.WasPassed()) {
+      const AuthenticationExtensionsLargeBlobOutputs& src =
+          mClientExtensionOutputs.mLargeBlob.Value();
+      AuthenticationExtensionsLargeBlobOutputsJSON& dest =
+          json.mClientExtensionResults.mLargeBlob.Construct();
+      if (src.mSupported.WasPassed()) {
+        dest.mSupported.Construct(src.mSupported.Value());
+      }
+    }
     json.mType.Assign(u"public-key"_ns);
     if (!ToJSValue(aCx, json, &value)) {
       aError.StealExceptionFromJSContext(aCx);
@@ -360,6 +403,53 @@ void PublicKeyCredential::ToJSON(JSContext* aCx,
     if (mClientExtensionOutputs.mPrf.WasPassed()) {
       json.mClientExtensionResults.mPrf.Construct();
     }
+    if (mClientExtensionOutputs.mPrf.WasPassed() && mPrfResultsFirst.isSome()) {
+      AuthenticationExtensionsPRFValuesJSON& dest =
+          json.mClientExtensionResults.mPrf.Value().mResults.Construct();
+      nsCString prfFirst;
+      nsresult rv = mozilla::Base64URLEncode(
+          mPrfResultsFirst->Length(), mPrfResultsFirst->Elements(),
+          Base64URLEncodePaddingPolicy::Omit, prfFirst);
+      if (NS_FAILED(rv)) {
+        aError.ThrowEncodingError(
+            "could not encode first prf output as urlsafe base64");
+        return;
+      }
+      dest.mFirst.Assign(NS_ConvertUTF8toUTF16(prfFirst));
+      if (mPrfResultsSecond.isSome()) {
+        nsCString prfSecond;
+        nsresult rv = mozilla::Base64URLEncode(
+            mPrfResultsSecond->Length(), mPrfResultsSecond->Elements(),
+            Base64URLEncodePaddingPolicy::Omit, prfSecond);
+        if (NS_FAILED(rv)) {
+          aError.ThrowEncodingError(
+              "could not encode second prf output as urlsafe base64");
+          return;
+        }
+        dest.mSecond.Construct(NS_ConvertUTF8toUTF16(prfSecond));
+      }
+    }
+    if (mClientExtensionOutputs.mLargeBlob.WasPassed()) {
+      const AuthenticationExtensionsLargeBlobOutputs& src =
+          mClientExtensionOutputs.mLargeBlob.Value();
+      AuthenticationExtensionsLargeBlobOutputsJSON& dest =
+          json.mClientExtensionResults.mLargeBlob.Construct();
+      if (src.mWritten.WasPassed()) {
+        dest.mWritten.Construct(src.mWritten.Value());
+      }
+      if (mLargeBlobValue.isSome()) {
+        nsCString largeBlobB64;
+        nsresult rv = mozilla::Base64URLEncode(
+            mLargeBlobValue->Length(), mLargeBlobValue->Elements(),
+            Base64URLEncodePaddingPolicy::Omit, largeBlobB64);
+        if (NS_FAILED(rv)) {
+          aError.ThrowEncodingError(
+              "could not encode large blob data as urlsafe base64");
+          return;
+        }
+        dest.mBlob.Construct(NS_ConvertUTF8toUTF16(largeBlobB64));
+      }
+    }
     json.mType.Assign(u"public-key"_ns);
     if (!ToJSValue(aCx, json, &value)) {
       aError.StealExceptionFromJSContext(aCx);
@@ -390,6 +480,28 @@ void PublicKeyCredential::SetClientExtensionResultHmacSecret(
   mClientExtensionOutputs.mHmacCreateSecret.Value() = aHmacCreateSecret;
 }
 
+void PublicKeyCredential::InitClientExtensionResultLargeBlob() {
+  mClientExtensionOutputs.mLargeBlob.Construct();
+}
+
+void PublicKeyCredential::SetClientExtensionResultLargeBlobSupported(
+    bool aLargeBlobSupported) {
+  mClientExtensionOutputs.mLargeBlob.Value().mSupported.Construct(
+      aLargeBlobSupported);
+}
+
+void PublicKeyCredential::SetClientExtensionResultLargeBlobValue(
+    const nsTArray<uint8_t>& aLargeBlobValue) {
+  mLargeBlobValue.emplace(aLargeBlobValue.Length());
+  mLargeBlobValue->Assign(aLargeBlobValue);
+}
+
+void PublicKeyCredential::SetClientExtensionResultLargeBlobWritten(
+    bool aLargeBlobWritten) {
+  mClientExtensionOutputs.mLargeBlob.Value().mWritten.Construct(
+      aLargeBlobWritten);
+}
+
 void PublicKeyCredential::InitClientExtensionResultPrf() {
   mClientExtensionOutputs.mPrf.Construct();
 }
@@ -474,6 +586,26 @@ bool DecodeAuthenticationExtensionsPRFInputsJSON(
   return true;
 }
 
+bool DecodeAuthenticationExtensionsLargeBlobInputsJSON(
+    GlobalObject& aGlobal,
+    const AuthenticationExtensionsLargeBlobInputsJSON& aInputsJSON,
+    AuthenticationExtensionsLargeBlobInputs& aInputs, ErrorResult& aRv) {
+  if (aInputsJSON.mSupport.WasPassed()) {
+    aInputs.mSupport.Construct(aInputsJSON.mSupport.Value());
+  }
+  if (aInputsJSON.mRead.WasPassed()) {
+    aInputs.mRead.Construct(aInputsJSON.mRead.Value());
+  }
+  if (aInputsJSON.mWrite.WasPassed()) {
+    if (!Base64DecodeToArrayBuffer(
+            aGlobal, aInputsJSON.mWrite.Value(),
+            aInputs.mWrite.Construct().SetAsArrayBuffer(), aRv)) {
+      return false;
+    }
+  }
+  return true;
+}
+
 void PublicKeyCredential::ParseCreationOptionsFromJSON(
     GlobalObject& aGlobal,
     const PublicKeyCredentialCreationOptionsJSON& aOptions,
@@ -528,30 +660,51 @@ void PublicKeyCredential::ParseCreationOptionsFromJSON(
     aResult.mAuthenticatorSelection = aOptions.mAuthenticatorSelection.Value();
   }
 
+  aResult.mHints = aOptions.mHints;
+
   aResult.mAttestation = aOptions.mAttestation;
 
   if (aOptions.mExtensions.WasPassed()) {
-    if (aOptions.mExtensions.Value().mAppid.WasPassed()) {
-      aResult.mExtensions.mAppid.Construct(
-          aOptions.mExtensions.Value().mAppid.Value());
+    const AuthenticationExtensionsClientInputsJSON& extInputsJSON =
+        aOptions.mExtensions.Value();
+    AuthenticationExtensionsClientInputs& extInputs = aResult.mExtensions;
+    if (extInputsJSON.mAppid.WasPassed()) {
+      extInputs.mAppid.Construct(extInputsJSON.mAppid.Value());
     }
-    if (aOptions.mExtensions.Value().mCredProps.WasPassed()) {
-      aResult.mExtensions.mCredProps.Construct(
-          aOptions.mExtensions.Value().mCredProps.Value());
+    if (extInputsJSON.mCredentialProtectionPolicy.WasPassed()) {
+      extInputs.mCredentialProtectionPolicy.Construct(
+          extInputsJSON.mCredentialProtectionPolicy.Value());
     }
-    if (aOptions.mExtensions.Value().mHmacCreateSecret.WasPassed()) {
-      aResult.mExtensions.mHmacCreateSecret.Construct(
-          aOptions.mExtensions.Value().mHmacCreateSecret.Value());
+    if (extInputsJSON.mEnforceCredentialProtectionPolicy.WasPassed()) {
+      extInputs.mEnforceCredentialProtectionPolicy.Construct(
+          extInputsJSON.mEnforceCredentialProtectionPolicy.Value());
     }
-    if (aOptions.mExtensions.Value().mMinPinLength.WasPassed()) {
-      aResult.mExtensions.mMinPinLength.Construct(
-          aOptions.mExtensions.Value().mMinPinLength.Value());
+    if (extInputsJSON.mCredProps.WasPassed()) {
+      extInputs.mCredProps.Construct(extInputsJSON.mCredProps.Value());
     }
-    if (aOptions.mExtensions.Value().mPrf.WasPassed()) {
+    if (extInputsJSON.mHmacCreateSecret.WasPassed()) {
+      extInputs.mHmacCreateSecret.Construct(
+          extInputsJSON.mHmacCreateSecret.Value());
+    }
+    if (extInputsJSON.mMinPinLength.WasPassed()) {
+      extInputs.mMinPinLength.Construct(extInputsJSON.mMinPinLength.Value());
+    }
+    if (extInputsJSON.mLargeBlob.WasPassed()) {
+      const AuthenticationExtensionsLargeBlobInputsJSON& largeBlobInputsJSON =
+          extInputsJSON.mLargeBlob.Value();
+      AuthenticationExtensionsLargeBlobInputs& largeBlobInputs =
+          aResult.mExtensions.mLargeBlob.Construct();
+      if (!DecodeAuthenticationExtensionsLargeBlobInputsJSON(
+              aGlobal, largeBlobInputsJSON, largeBlobInputs, aRv)) {
+        aRv.ThrowEncodingError(
+            "could not decode large blob inputs as urlsafe base64");
+        return;
+      }
+    }
+    if (extInputsJSON.mPrf.WasPassed()) {
       const AuthenticationExtensionsPRFInputsJSON& prfInputsJSON =
-          aOptions.mExtensions.Value().mPrf.Value();
-      AuthenticationExtensionsPRFInputs& prfInputs =
-          aResult.mExtensions.mPrf.Construct();
+          extInputsJSON.mPrf.Value();
+      AuthenticationExtensionsPRFInputs& prfInputs = extInputs.mPrf.Construct();
       if (!DecodeAuthenticationExtensionsPRFInputsJSON(aGlobal, prfInputsJSON,
                                                        prfInputs, aRv)) {
         aRv.ThrowEncodingError("could not decode prf inputs as urlsafe base64");
@@ -602,6 +755,8 @@ void PublicKeyCredential::ParseRequestOptionsFromJSON(
 
   aResult.mUserVerification = aOptions.mUserVerification;
 
+  aResult.mHints = aOptions.mHints;
+
   if (aOptions.mExtensions.WasPassed()) {
     if (aOptions.mExtensions.Value().mAppid.WasPassed()) {
       aResult.mExtensions.mAppid.Construct(
@@ -611,9 +766,17 @@ void PublicKeyCredential::ParseRequestOptionsFromJSON(
       aResult.mExtensions.mCredProps.Construct(
           aOptions.mExtensions.Value().mCredProps.Value());
     }
-    if (aOptions.mExtensions.Value().mHmacCreateSecret.WasPassed()) {
-      aResult.mExtensions.mHmacCreateSecret.Construct(
-          aOptions.mExtensions.Value().mHmacCreateSecret.Value());
+    if (aOptions.mExtensions.Value().mLargeBlob.WasPassed()) {
+      const AuthenticationExtensionsLargeBlobInputsJSON& largeBlobInputsJSON =
+          aOptions.mExtensions.Value().mLargeBlob.Value();
+      AuthenticationExtensionsLargeBlobInputs& largeBlobInputs =
+          aResult.mExtensions.mLargeBlob.Construct();
+      if (!DecodeAuthenticationExtensionsLargeBlobInputsJSON(
+              aGlobal, largeBlobInputsJSON, largeBlobInputs, aRv)) {
+        aRv.ThrowEncodingError(
+            "could not decode large blob inputs as urlsafe base64");
+        return;
+      }
     }
     if (aOptions.mExtensions.Value().mMinPinLength.WasPassed()) {
       aResult.mExtensions.mMinPinLength.Construct(
