@@ -1140,7 +1140,7 @@ void txMozillaXSLTProcessor::notifyError() {
 
   IgnoredErrorResult rv;
   ElementCreationOptionsOrString options;
-  Unused << options.SetAsString();
+  (void)options.SetAsString();
 
   nsCOMPtr<Element> element =
       document->CreateElementNS(ns, u"parsererror"_ns, options, rv);
@@ -1162,7 +1162,7 @@ void txMozillaXSLTProcessor::notifyError() {
 
   if (!mSourceText.IsEmpty()) {
     ElementCreationOptionsOrString options;
-    Unused << options.SetAsString();
+    (void)options.SetAsString();
 
     nsCOMPtr<Element> sourceElement =
         document->CreateElementNS(ns, u"sourcetext"_ns, options, rv);
@@ -1228,8 +1228,7 @@ void txMozillaXSLTProcessor::CharacterDataChanged(
 
 void txMozillaXSLTProcessor::AttributeChanged(Element* aElement,
                                               int32_t aNameSpaceID,
-                                              nsAtom* aAttribute,
-                                              int32_t aModType,
+                                              nsAtom* aAttribute, AttrModType,
                                               const nsAttrValue* aOldValue) {
   mStylesheet = nullptr;
 }
