@@ -34,8 +34,6 @@
 #include "mozilla/dom/BrowserChild.h"
 #include "mozilla/dom/BrowserParent.h"
 #include "mozilla/dom/ContentChild.h"
-#include "mozilla/Unused.h"
-#include "mozilla/DebugOnly.h"
 #include "mozilla/SpinEventLoopUntil.h"
 #include "nsThreadUtils.h"
 #if defined(XP_WIN)
@@ -47,7 +45,6 @@
 #endif
 #include "VsyncSource.h"
 
-using mozilla::Unused;
 using mozilla::gfx::GPUProcessManager;
 
 namespace mozilla {
@@ -165,7 +162,7 @@ void CompositorBridgeChild::Destroy() {
   AutoTArray<PAPZChild*, 16> apzChildren;
   ManagedPAPZChild(apzChildren);
   for (PAPZChild* child : apzChildren) {
-    Unused << child->SendDestroy();
+    (void)child->SendDestroy();
   }
 
   const ManagedContainer<PTextureChild>& textures = ManagedPTextureChild();
@@ -440,7 +437,7 @@ mozilla::ipc::IPCResult CompositorBridgeChild::RecvCompositorOptionsChanged(
 
   if (RefPtr<dom::BrowserParent> tab =
           dom::BrowserParent::GetBrowserParentFromLayersId(aLayersId)) {
-    Unused << tab->SendCompositorOptionsChanged(aNewOptions);
+    (void)tab->SendCompositorOptionsChanged(aNewOptions);
   }
   return IPC_OK();
 }
