@@ -15,6 +15,7 @@
 #include "jsfriendapi.h"
 #include "js/Modules.h"       // JS::{Get,Set}ModulePrivate
 #include "LoadContextBase.h"  // LoadContextBase
+#include "nsIChannel.h"       // nsIChannel
 
 namespace JS::loader {
 
@@ -28,7 +29,19 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(LoadedScript)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_CYCLE_COLLECTION(LoadedScript, mFetchOptions, mURI, mBaseURL)
+// LoadedScript can be accessed from multiple threads.
+//
+// For instance, worker script loader passes the ScriptLoadRequest and
+// the associated LoadedScript to the main thread to perform the actual load.
+// Even while it's handled by the main thread, the LoadedScript is
+// the target of the worker thread's cycle collector.
+//
+// Fields that can be modified by other threads shouldn't be touched by
+// the cycle collection.
+//
+// NOTE: nsIURI doesn't have to be touched here because it cannot be a part
+//       of cycle.
+NS_IMPL_CYCLE_COLLECTION(LoadedScript, mFetchOptions, mCacheInfo)
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(LoadedScript)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(LoadedScript)
@@ -36,35 +49,35 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE(LoadedScript)
 LoadedScript::LoadedScript(ScriptKind aKind,
                            mozilla::dom::ReferrerPolicy aReferrerPolicy,
                            ScriptFetchOptions* aFetchOptions, nsIURI* aURI)
-    : mKind(aKind),
+    : mDataType(DataType::eUnknown),
+      mKind(aKind),
       mReferrerPolicy(aReferrerPolicy),
+      mBytecodeOffset(0),
       mFetchOptions(aFetchOptions),
       mURI(aURI),
-      mDataType(DataType::eUnknown),
-      mReceivedScriptTextLength(0),
-      mBytecodeOffset(0) {
+      mReceivedScriptTextLength(0) {
   MOZ_ASSERT(mFetchOptions);
   MOZ_ASSERT(mURI);
 }
 
 LoadedScript::LoadedScript(const LoadedScript& aOther)
-    : mKind(aOther.mKind),
+    : mDataType(DataType::eCachedStencil),
+      mKind(aOther.mKind),
       mReferrerPolicy(aOther.mReferrerPolicy),
+      mBytecodeOffset(0),
       mFetchOptions(aOther.mFetchOptions),
       mURI(aOther.mURI),
       mBaseURL(aOther.mBaseURL),
-      mDataType(DataType::eStencil),
       mReceivedScriptTextLength(0),
-      mBytecodeOffset(0),
       mStencil(aOther.mStencil) {
   MOZ_ASSERT(mFetchOptions);
   MOZ_ASSERT(mURI);
   // NOTE: This is only for the stencil case.
   //       The script text and the bytecode are not reflected.
-  MOZ_DIAGNOSTIC_ASSERT(aOther.mDataType == DataType::eStencil);
+  MOZ_DIAGNOSTIC_ASSERT(aOther.mDataType == DataType::eCachedStencil);
   MOZ_DIAGNOSTIC_ASSERT(mStencil);
   MOZ_ASSERT(!mScriptData);
-  MOZ_ASSERT(mScriptBytecode.empty());
+  MOZ_ASSERT(mSRIAndBytecode.empty());
 }
 
 LoadedScript::~LoadedScript() {
@@ -117,7 +130,7 @@ size_t LoadedScript::SizeOfIncludingThis(
     }
   }
 
-  bytes += mScriptBytecode.sizeOfExcludingThis(aMallocSizeOf);
+  bytes += mSRIAndBytecode.sizeOfExcludingThis(aMallocSizeOf);
 
   // NOTE: Stencil is reported by SpiderMonkey.
   return bytes;
@@ -198,6 +211,22 @@ nsresult LoadedScript::GetScriptSource(JSContext* aCx,
   return NS_OK;
 }
 
+static bool IsInternalURIScheme(nsIURI* uri) {
+  return uri->SchemeIs("moz-extension") || uri->SchemeIs("resource") ||
+         uri->SchemeIs("moz-src") || uri->SchemeIs("chrome");
+}
+
+void LoadedScript::SetBaseURLFromChannelAndOriginalURI(nsIChannel* aChannel,
+                                                       nsIURI* aOriginalURI) {
+  // Fixup moz-extension: and resource: URIs, because the channel URI will
+  // point to file:, which won't be allowed to load.
+  if (aOriginalURI && IsInternalURIScheme(aOriginalURI)) {
+    mBaseURL = aOriginalURI;
+  } else {
+    aChannel->GetURI(getter_AddRefs(mBaseURL));
+  }
+}
+
 inline void CheckModuleScriptPrivate(LoadedScript* script,
                                      const Value& aPrivate) {
 #ifdef DEBUG
@@ -249,6 +278,16 @@ ClassicScript::ClassicScript(mozilla::dom::ReferrerPolicy aReferrerPolicy,
     : LoadedScript(ScriptKind::eClassic, aReferrerPolicy, aFetchOptions, aURI) {
 }
 
+//////////////////////////////////////////////////////////////
+// ImportMapScript
+//////////////////////////////////////////////////////////////
+
+ImportMapScript::ImportMapScript(mozilla::dom::ReferrerPolicy aReferrerPolicy,
+                                 ScriptFetchOptions* aFetchOptions,
+                                 nsIURI* aURI)
+    : LoadedScript(ScriptKind::eImportMap, aReferrerPolicy, aFetchOptions,
+                   aURI) {}
+
 //////////////////////////////////////////////////////////////
 // ModuleScript
 //////////////////////////////////////////////////////////////
@@ -261,6 +300,7 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(ModuleScript, LoadedScript)
   tmp->UnlinkModuleRecord();
   tmp->mParseError.setUndefined();
   tmp->mErrorToRethrow.setUndefined();
+  tmp->DropDiskCacheReference();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(ModuleScript, LoadedScript)
@@ -290,13 +330,13 @@ ModuleScript::ModuleScript(const LoadedScript& aOther) : LoadedScript(aOther) {
 already_AddRefed<ModuleScript> ModuleScript::FromCache(
     const LoadedScript& aScript) {
   MOZ_DIAGNOSTIC_ASSERT(aScript.IsModuleScript());
-  MOZ_DIAGNOSTIC_ASSERT(aScript.IsStencil());
+  MOZ_DIAGNOSTIC_ASSERT(aScript.IsCachedStencil());
 
   return mozilla::MakeRefPtr<ModuleScript>(aScript).forget();
 }
 
 already_AddRefed<LoadedScript> ModuleScript::ToCache() {
-  MOZ_DIAGNOSTIC_ASSERT(IsStencil());
+  MOZ_DIAGNOSTIC_ASSERT(IsCachedStencil());
   MOZ_DIAGNOSTIC_ASSERT(!HasParseError());
   MOZ_DIAGNOSTIC_ASSERT(!HasErrorToRethrow());
 
