@@ -142,11 +142,11 @@ static bool CreateBadModuleTypeError(JSContext* aCx, LoadedScript* aScript,
 
 // https://html.spec.whatwg.org/#hostloadimportedmodule
 // static
-bool ModuleLoaderBase::HostLoadImportedModule(JSContext* aCx,
-                                              Handle<JSScript*> aReferrer,
-                                              Handle<JSObject*> aModuleRequest,
-                                              Handle<Value> aHostDefined,
-                                              Handle<Value> aPayload) {
+bool ModuleLoaderBase::HostLoadImportedModule(
+    JSContext* aCx, Handle<JSScript*> aReferrer,
+    Handle<JSObject*> aModuleRequest, Handle<Value> aHostDefined,
+    Handle<Value> aPayload, uint32_t aLineNumber,
+    JS::ColumnNumberOneOrigin aColumnNumber) {
   Rooted<JSObject*> object(aCx);
   if (aPayload.isObject()) {
     object = &aPayload.toObject();
@@ -195,7 +195,7 @@ bool ModuleLoaderBase::HostLoadImportedModule(JSContext* aCx,
       Rooted<Value> error(aCx);
       nsresult rv =
           loader->HandleResolveFailure(aCx, script, string, result.unwrapErr(),
-                                       0, ColumnNumberOneOrigin(), &error);
+                                       aLineNumber, aColumnNumber, &error);
       if (NS_FAILED(rv)) {
         JS_ReportOutOfMemory(aCx);
         return false;
@@ -538,22 +538,22 @@ nsresult ModuleLoaderBase::RestartModuleLoad(ModuleLoadRequest* aRequest) {
 nsresult ModuleLoaderBase::StartOrRestartModuleLoad(ModuleLoadRequest* aRequest,
                                                     RestartRequest aRestart) {
   MOZ_ASSERT(aRequest->mLoader == this);
-  MOZ_ASSERT(aRequest->IsFetching() || aRequest->IsPendingFetchingError());
+  MOZ_ASSERT(aRequest->IsFetching());
 
   // NOTE: The LoadedScript::mDataType field used by the IsStencil call can be
   //       modified asynchronously after the StartFetch call.
   //       In order to avoid the race condition, cache the value here.
-  bool isStencil = aRequest->IsStencil();
+  bool isCachedStencil = aRequest->IsCachedStencil();
 
-  MOZ_ASSERT_IF(isStencil, aRestart == RestartRequest::No);
+  MOZ_ASSERT_IF(isCachedStencil, aRestart == RestartRequest::No);
 
-  if (!isStencil) {
+  if (!isCachedStencil) {
     aRequest->SetUnknownDataType();
   }
 
   if (LOG_ENABLED()) {
     nsAutoCString url;
-    aRequest->mURI->GetAsciiSpec(url);
+    aRequest->URI()->GetAsciiSpec(url);
     LOG(("ScriptLoadRequest (%p): Start module load %s", aRequest, url.get()));
   }
 
@@ -561,7 +561,7 @@ nsresult ModuleLoaderBase::StartOrRestartModuleLoad(ModuleLoadRequest* aRequest,
   // "fetching" map.
   MOZ_ASSERT_IF(
       aRestart == RestartRequest::Yes,
-      IsModuleFetching(ModuleMapKey(aRequest->mURI, aRequest->mModuleType)));
+      IsModuleFetching(ModuleMapKey(aRequest->URI(), aRequest->mModuleType)));
 
   // Check with the derived class whether we should load this module.
   nsresult rv = NS_OK;
@@ -573,7 +573,7 @@ nsresult ModuleLoaderBase::StartOrRestartModuleLoad(ModuleLoadRequest* aRequest,
   // and if so wait for it rather than starting a new fetch.
   if (aRestart == RestartRequest::No &&
       ModuleMapContainsURL(
-          ModuleMapKey(aRequest->mURI, aRequest->mModuleType))) {
+          ModuleMapKey(aRequest->URI(), aRequest->mModuleType))) {
     LOG(("ScriptLoadRequest (%p): Waiting for module fetch", aRequest));
     WaitForModuleFetch(aRequest);
     return NS_OK;
@@ -582,9 +582,9 @@ nsresult ModuleLoaderBase::StartOrRestartModuleLoad(ModuleLoadRequest* aRequest,
   rv = StartFetch(aRequest);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (isStencil) {
+  if (isCachedStencil) {
     MOZ_ASSERT(
-        IsModuleFetched(ModuleMapKey(aRequest->mURI, aRequest->mModuleType)));
+        IsModuleFetched(ModuleMapKey(aRequest->URI(), aRequest->mModuleType)));
     return NS_OK;
   }
 
@@ -626,9 +626,9 @@ nsresult ModuleLoaderBase::GetFetchedModuleURLs(nsTArray<nsCString>& aURLs) {
 void ModuleLoaderBase::SetModuleFetchStarted(ModuleLoadRequest* aRequest) {
   // Update the module map to indicate that a module is currently being fetched.
 
-  ModuleMapKey moduleMapKey(aRequest->mURI, aRequest->mModuleType);
+  ModuleMapKey moduleMapKey(aRequest->URI(), aRequest->mModuleType);
 
-  MOZ_ASSERT(aRequest->IsFetching() || aRequest->IsPendingFetchingError());
+  MOZ_ASSERT(aRequest->IsFetching());
   MOZ_ASSERT(!ModuleMapContainsURL(moduleMapKey));
 
   RefPtr<LoadingRequest> loadingRequest = new LoadingRequest();
@@ -651,7 +651,7 @@ ModuleLoaderBase::SetModuleFetchFinishedAndGetWaitingRequests(
        "%u)",
        aRequest, aRequest->mModuleScript.get(), unsigned(aResult)));
 
-  ModuleMapKey moduleMapKey(aRequest->mURI, aRequest->mModuleType);
+  ModuleMapKey moduleMapKey(aRequest->URI(), aRequest->mModuleType);
 
   auto entry = mFetchingModules.Lookup(moduleMapKey);
   if (!entry) {
@@ -707,7 +707,7 @@ void ModuleLoaderBase::ResumeWaitingRequest(ModuleLoadRequest* aRequest,
 }
 
 void ModuleLoaderBase::WaitForModuleFetch(ModuleLoadRequest* aRequest) {
-  ModuleMapKey moduleMapKey(aRequest->mURI, aRequest->mModuleType);
+  ModuleMapKey moduleMapKey(aRequest->URI(), aRequest->mModuleType);
   MOZ_ASSERT(ModuleMapContainsURL(moduleMapKey));
 
   if (auto entry = mFetchingModules.Lookup(moduleMapKey)) {
@@ -866,7 +866,7 @@ void ModuleLoaderBase::OnFetchFailed(ModuleLoadRequest* aRequest) {
     // cases.
     if (aRequest->GetRootModule()->IsDynamicImport()) {
       nsAutoCString url;
-      aRequest->mURI->GetSpec(url);
+      aRequest->URI()->GetSpec(url);
       JS_ReportErrorNumberASCII(cx, js::GetErrorMessage, nullptr,
                                 JSMSG_DYNAMIC_IMPORT_FAILED, url.get());
       FinishLoadingImportedModuleFailedWithPendingException(cx, payload);
@@ -962,7 +962,7 @@ void ModuleLoaderBase::DispatchModuleErrored(ModuleLoadRequest* aRequest) {
 
 nsresult ModuleLoaderBase::CreateModuleScript(ModuleLoadRequest* aRequest) {
   MOZ_ASSERT(!aRequest->mModuleScript);
-  MOZ_ASSERT(aRequest->mBaseURL);
+  MOZ_ASSERT(aRequest->BaseURL());
 
   LOG(("ScriptLoadRequest (%p): Create module script", aRequest));
 
@@ -1002,29 +1002,6 @@ nsresult ModuleLoaderBase::CreateModuleScript(ModuleLoadRequest* aRequest) {
     }
 
     MOZ_ASSERT(aRequest->mLoadedScript->IsModuleScript());
-    MOZ_ASSERT(aRequest->mFetchOptions->IsCompatible(
-        aRequest->mLoadedScript->GetFetchOptions()));
-#ifdef DEBUG
-    {
-      bool equals = false;
-      aRequest->mURI->Equals(aRequest->mLoadedScript->GetURI(), &equals);
-      MOZ_ASSERT(equals);
-    }
-#endif
-
-    if (!aRequest->mLoadedScript->BaseURL()) {
-      // If this script is not cached, the BaseURL should be copied from
-      // request to script for later use.
-      aRequest->mLoadedScript->SetBaseURL(aRequest->mBaseURL);
-    } else {
-      // If this script is cached, the BaseURL should match, which is
-      // checked when looking for the cache.
-#ifdef DEBUG
-      bool equals = false;
-      aRequest->mBaseURL->Equals(aRequest->mLoadedScript->BaseURL(), &equals);
-      MOZ_ASSERT(equals);
-#endif
-    }
     RefPtr<ModuleScript> moduleScript =
         aRequest->mLoadedScript->AsModuleScript();
 
@@ -1054,19 +1031,6 @@ nsresult ModuleLoaderBase::CreateModuleScript(ModuleLoadRequest* aRequest) {
     }
 
     moduleScript->SetModuleRecord(module);
-
-    // TODO: Bug 1968885: Remove ModuleLoaderBase::ResolveRequestedModules
-    //
-    // Validate requested modules and treat failure to resolve module specifiers
-    // the same as a parse error.
-    rv = ResolveRequestedModules(aRequest, nullptr);
-    if (NS_FAILED(rv)) {
-      if (!aRequest->IsErrored()) {
-        aRequest->mModuleScript = nullptr;
-        return rv;
-      }
-      return NS_OK;
-    }
   }
 
   LOG(("ScriptLoadRequest (%p):   module script == %p", aRequest,
@@ -1135,76 +1099,6 @@ ResolveResult ModuleLoaderBase::ResolveModuleSpecifier(
                                            aSpecifier);
 }
 
-nsresult ModuleLoaderBase::ResolveRequestedModules(
-    ModuleLoadRequest* aRequest, nsTArray<ModuleMapKey>* aRequestedModulesOut) {
-  MOZ_ASSERT_IF(aRequestedModulesOut, aRequestedModulesOut->IsEmpty());
-
-  ModuleScript* ms = aRequest->mModuleScript;
-
-  AutoJSAPI jsapi;
-  if (!jsapi.Init(mGlobalObject)) {
-    return NS_ERROR_FAILURE;
-  }
-
-  JSContext* cx = jsapi.cx();
-  Rooted<JSObject*> moduleRecord(cx, ms->ModuleRecord());
-
-  if (!IsCyclicModule(moduleRecord)) {
-    return NS_OK;
-  }
-
-  uint32_t length = GetRequestedModulesCount(cx, moduleRecord);
-
-  for (uint32_t i = 0; i < length; i++) {
-    Rooted<JSString*> str(cx, GetRequestedModuleSpecifier(cx, moduleRecord, i));
-    if (!str) {
-      Rooted<Value> pendingException(cx);
-      if (!JS_GetPendingException(cx, &pendingException)) {
-        return NS_ERROR_FAILURE;
-      }
-      ms->SetParseError(pendingException);
-      JS_ClearPendingException(cx);
-      return NS_ERROR_FAILURE;
-    }
-
-    nsAutoJSString specifier;
-    if (!specifier.init(cx, str)) {
-      return NS_ERROR_FAILURE;
-    }
-
-    // Let url be the result of resolving a module specifier given module script
-    // and requested.
-    ModuleLoaderBase* loader = aRequest->mLoader;
-    auto result = loader->ResolveModuleSpecifier(ms, specifier);
-    if (result.isErr()) {
-      uint32_t lineNumber = 0;
-      ColumnNumberOneOrigin columnNumber;
-      GetRequestedModuleSourcePos(cx, moduleRecord, i, &lineNumber,
-                                  &columnNumber);
-
-      Rooted<Value> error(cx);
-      nsresult rv =
-          loader->HandleResolveFailure(cx, ms, specifier, result.unwrapErr(),
-                                       lineNumber, columnNumber, &error);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      ms->SetParseError(error);
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIURI> uri = result.unwrap();
-    if (aRequestedModulesOut) {
-      // Let moduleType be the result of running the module type from module
-      // request steps given moduleRequest.
-      ModuleType moduleType = GetRequestedModuleType(cx, moduleRecord, i);
-
-      aRequestedModulesOut->AppendElement(ModuleMapKey(uri, moduleType));
-    }
-  }
-
-  return NS_OK;
-}
-
 void ModuleLoaderBase::StartFetchingModuleDependencies(
     ModuleLoadRequest* aRequest) {
   if (aRequest->IsCanceled()) {
@@ -1242,8 +1136,8 @@ void ModuleLoaderBase::StartFetchingModuleDependencies(
   // destroyed. As a result, the promise returned by LoadRequestedModules may
   // neither resolve nor reject. To ensure module loading completes reliably in
   // chrome pages, we use the synchronous variant of LoadRequestedModules.
-  bool isSync = aRequest->mURI->SchemeIs("chrome") ||
-                aRequest->mURI->SchemeIs("resource");
+  bool isSync = aRequest->URI()->SchemeIs("chrome") ||
+                aRequest->URI()->SchemeIs("resource");
 
   // TODO: Bug1973660: Use Promise version of LoadRequestedModules on Workers.
   if (aRequest->HasScriptLoadContext() && !isSync) {
@@ -1365,7 +1259,7 @@ bool ModuleLoaderBase::OnLoadRequestedModulesRejected(
       FinishLoadingImportedModuleFailed(aCx, payload, error);
     } else {
       nsAutoCString url;
-      aRequest->mURI->GetSpec(url);
+      aRequest->URI()->GetSpec(url);
       JS_ReportErrorNumberASCII(aCx, js::GetErrorMessage, nullptr,
                                 JSMSG_DYNAMIC_IMPORT_FAILED, url.get());
       FinishLoadingImportedModuleFailedWithPendingException(aCx, payload);
@@ -1571,6 +1465,10 @@ void ModuleLoaderBase::ProcessDynamicImport(ModuleLoadRequest* aRequest) {
 
   LOG(("ScriptLoadRequest (%p): ProcessDynamicImport", aRequest));
   FinishLoadingImportedModule(cx, aRequest);
+
+  (void)mLoader->MaybePrepareModuleForDiskCacheAfterExecute(aRequest, NS_OK);
+
+  mLoader->MaybeUpdateDiskCache();
 }
 
 nsresult ModuleLoaderBase::EvaluateModule(ModuleLoadRequest* aRequest) {
@@ -1626,14 +1524,11 @@ nsresult ModuleLoaderBase::EvaluateModuleInContext(
   }
 
   if (aRequest->HasScriptLoadContext()) {
-    TRACE_FOR_TEST(aRequest, "scriptloader_evaluate_module");
+    TRACE_FOR_TEST(aRequest, "evaluate:module");
   }
 
   Rooted<Value> rval(aCx);
 
-  // TODO: Bug 1973321: Prepare Bytecode encoding for dynamic import
-  mLoader->MaybePrepareModuleForCacheBeforeExecute(aCx, aRequest);
-
   bool ok = ModuleEvaluate(aCx, module, &rval);
 
   // ModuleEvaluate will usually set a pending exception if it returns false,
@@ -1661,9 +1556,10 @@ nsresult ModuleLoaderBase::EvaluateModuleInContext(
     LOG(("ScriptLoadRequest (%p):   evaluation failed on throw", aRequest));
   }
 
-  rv = mLoader->MaybePrepareModuleForCacheAfterExecute(aRequest, NS_OK);
+  // TODO: Bug 1973321: Prepare Bytecode encoding for dynamic import
+  rv = mLoader->MaybePrepareModuleForDiskCacheAfterExecute(aRequest, NS_OK);
 
-  mLoader->MaybeUpdateCache();
+  mLoader->MaybeUpdateDiskCache();
 
   return rv;
 }
@@ -1710,7 +1606,7 @@ UniquePtr<ImportMap> ModuleLoaderBase::ParseImportMap(
   // supported, therefore parsing and registering import-maps will be executed
   // consecutively. To simplify the implementation, we didn't create the 'error
   // to rethow' item and report the exception immediately(done in ~AutoJSAPI).
-  return ImportMap::ParseString(jsapi.cx(), text, aRequest->mBaseURL, warning);
+  return ImportMap::ParseString(jsapi.cx(), text, aRequest->BaseURL(), warning);
 }
 
 void ModuleLoaderBase::RegisterImportMap(UniquePtr<ImportMap> aImportMap) {
