@@ -10,7 +10,6 @@
 #include "mozilla/DebugOnly.h"
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/MemoryReporting.h"
-#include "mozilla/ThreadLocal.h"
 
 #include "gc/GCContext.h"
 #include "gc/PublicIterators.h"
@@ -177,24 +176,6 @@ bool JitRuntime::generateTrampolines(JSContext* cx) {
   generateInvalidator(masm, &bailoutTail);
   rangeRecorder.recordOffset("Trampoline: Invalidator");
 
-  // The arguments rectifier has to use the same frame layout as the function
-  // frames it rectifies.
-  static_assert(std::is_base_of_v<JitFrameLayout, RectifierFrameLayout>,
-                "a rectifier frame can be used with jit frame");
-  static_assert(std::is_base_of_v<JitFrameLayout, WasmToJSJitFrameLayout>,
-                "wasm frames simply are jit frames");
-  static_assert(sizeof(JitFrameLayout) == sizeof(WasmToJSJitFrameLayout),
-                "thus a rectifier frame can be used with a wasm frame");
-
-  JitSpew(JitSpew_Codegen, "# Emitting arguments rectifier");
-  generateArgumentsRectifier(masm, ArgumentsRectifierKind::Normal);
-  rangeRecorder.recordOffset("Trampoline: Arguments Rectifier");
-
-  JitSpew(JitSpew_Codegen, "# Emitting trial inlining arguments rectifier");
-  generateArgumentsRectifier(masm, ArgumentsRectifierKind::TrialInlining);
-  rangeRecorder.recordOffset(
-      "Trampoline: Arguments Rectifier (Trial Inlining)");
-
   JitSpew(JitSpew_Codegen, "# Emitting EnterJIT sequence");
   generateEnterJIT(cx, masm);
   rangeRecorder.recordOffset("Trampoline: EnterJIT");
@@ -975,13 +956,12 @@ namespace jit {
 
 bool OptimizeMIR(MIRGenerator* mir) {
   MIRGraph& graph = mir->graph();
-  GraphSpewer& gs = mir->graphSpewer();
 
   if (mir->shouldCancel("Start")) {
     return false;
   }
 
-  gs.spewPass("BuildSSA");
+  mir->spewPass("BuildSSA");
   AssertBasicGraphCoherency(graph);
 
   if (JitSpewEnabled(JitSpew_MIRExpressions)) {
@@ -995,7 +975,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!PruneUnusedBranches(mir, graph)) {
       return false;
     }
-    gs.spewPass("Prune Unused Branches");
+    mir->spewPass("Prune Unused Branches");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("Prune Unused Branches")) {
@@ -1008,7 +988,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!FoldEmptyBlocks(graph, &dummy)) {
       return false;
     }
-    gs.spewPass("Fold Empty Blocks");
+    mir->spewPass("Fold Empty Blocks");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("Fold Empty Blocks")) {
@@ -1022,7 +1002,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateTriviallyDeadResumePointOperands(mir, graph)) {
       return false;
     }
-    gs.spewPass("Eliminate trivially dead resume point operands");
+    mir->spewPass("Eliminate trivially dead resume point operands");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("Eliminate trivially dead resume point operands")) {
@@ -1034,7 +1014,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!FoldTests(graph)) {
       return false;
     }
-    gs.spewPass("Fold Tests");
+    mir->spewPass("Fold Tests");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("Fold Tests")) {
@@ -1046,7 +1026,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!SplitCriticalEdges(graph)) {
       return false;
     }
-    gs.spewPass("Split Critical Edges");
+    mir->spewPass("Split Critical Edges");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Split Critical Edges")) {
@@ -1056,7 +1036,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
 
   {
     RenumberBlocks(graph);
-    gs.spewPass("Renumber Blocks");
+    mir->spewPass("Renumber Blocks");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Renumber Blocks")) {
@@ -1086,7 +1066,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminatePhis(mir, graph, observability)) {
       return false;
     }
-    gs.spewPass("Eliminate phis");
+    mir->spewPass("Eliminate phis");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Eliminate phis")) {
@@ -1108,7 +1088,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!OptimizeIteratorIndices(mir, graph)) {
       return false;
     }
-    gs.spewPass("Iterator Indices");
+    mir->spewPass("Iterator Indices");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Iterator Indices")) {
@@ -1122,7 +1102,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!ScalarReplacement(mir, graph)) {
       return false;
     }
-    gs.spewPass("Scalar Replacement");
+    mir->spewPass("Scalar Replacement");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Scalar Replacement")) {
@@ -1134,7 +1114,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!ApplyTypeInformation(mir, graph)) {
       return false;
     }
-    gs.spewPass("Apply types");
+    mir->spewPass("Apply types");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Apply types")) {
@@ -1146,7 +1126,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!TrackWasmRefTypes(graph)) {
       return false;
     }
-    gs.spewPass("Track Wasm ref types");
+    mir->spewPass("Track Wasm ref types");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Track Wasm ref types")) {
@@ -1159,7 +1139,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!ama.analyze()) {
       return false;
     }
-    gs.spewPass("Alignment Mask Analysis");
+    mir->spewPass("Alignment Mask Analysis");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Alignment Mask Analysis")) {
@@ -1182,7 +1162,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
         return false;
       }
 
-      gs.spewPass("Alias analysis");
+      mir->spewPass("Alias analysis");
       AssertExtendedGraphCoherency(graph);
 
       if (mir->shouldCancel("Alias analysis")) {
@@ -1198,7 +1178,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
         return false;
       }
 
-      gs.spewPass("Eliminate dead resume point operands");
+      mir->spewPass("Eliminate dead resume point operands");
       AssertExtendedGraphCoherency(graph);
 
       if (mir->shouldCancel("Eliminate dead resume point operands")) {
@@ -1212,7 +1192,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!gvn.run(ValueNumberer::UpdateAliasAnalysis)) {
       return false;
     }
-    gs.spewPass("GVN");
+    mir->spewPass("GVN");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("GVN")) {
@@ -1225,7 +1205,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!BranchHinting(mir, graph)) {
       return false;
     }
-    gs.spewPass("BranchHinting");
+    mir->spewPass("BranchHinting");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("BranchHinting")) {
@@ -1241,7 +1221,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!LICM(mir, graph)) {
       return false;
     }
-    gs.spewPass("LICM");
+    mir->spewPass("LICM");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("LICM")) {
@@ -1255,7 +1235,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!r.addBetaNodes()) {
       return false;
     }
-    gs.spewPass("Beta");
+    mir->spewPass("Beta");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("RA Beta")) {
@@ -1265,7 +1245,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!r.analyze() || !r.addRangeAssertions()) {
       return false;
     }
-    gs.spewPass("Range Analysis");
+    mir->spewPass("Range Analysis");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Range Analysis")) {
@@ -1275,7 +1255,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!r.removeBetaNodes()) {
       return false;
     }
-    gs.spewPass("De-Beta");
+    mir->spewPass("De-Beta");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("RA De-Beta")) {
@@ -1287,7 +1267,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
       if (!r.prepareForUCE(&shouldRunUCE)) {
         return false;
       }
-      gs.spewPass("RA check UCE");
+      mir->spewPass("RA check UCE");
       AssertExtendedGraphCoherency(graph);
 
       if (mir->shouldCancel("RA check UCE")) {
@@ -1298,7 +1278,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
         if (!gvn.run(ValueNumberer::DontUpdateAliasAnalysis)) {
           return false;
         }
-        gs.spewPass("UCE After RA");
+        mir->spewPass("UCE After RA");
         AssertExtendedGraphCoherency(graph);
 
         if (mir->shouldCancel("UCE After RA")) {
@@ -1311,7 +1291,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
       if (!r.truncate()) {
         return false;
       }
-      gs.spewPass("Truncate Doubles");
+      mir->spewPass("Truncate Doubles");
       AssertExtendedGraphCoherency(graph);
 
       if (mir->shouldCancel("Truncate Doubles")) {
@@ -1325,7 +1305,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!Sink(mir, graph)) {
       return false;
     }
-    gs.spewPass("Sink");
+    mir->spewPass("Sink");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Sink")) {
@@ -1339,7 +1319,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!r.removeUnnecessaryBitops()) {
       return false;
     }
-    gs.spewPass("Remove Unnecessary Bitops");
+    mir->spewPass("Remove Unnecessary Bitops");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Remove Unnecessary Bitops")) {
@@ -1352,7 +1332,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!FoldLinearArithConstants(mir, graph)) {
       return false;
     }
-    gs.spewPass("Fold Linear Arithmetic Constants");
+    mir->spewPass("Fold Linear Arithmetic Constants");
     AssertBasicGraphCoherency(graph);
 
     if (mir->shouldCancel("Fold Linear Arithmetic Constants")) {
@@ -1367,7 +1347,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!eaa.analyze()) {
       return false;
     }
-    gs.spewPass("Effective Address Analysis");
+    mir->spewPass("Effective Address Analysis");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Effective Address Analysis")) {
@@ -1381,7 +1361,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateBoundsChecks(mir, graph)) {
       return false;
     }
-    gs.spewPass("Redundant Bounds Check Elimination");
+    mir->spewPass("Redundant Bounds Check Elimination");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("BCE")) {
@@ -1393,7 +1373,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateDeadCode(mir, graph)) {
       return false;
     }
-    gs.spewPass("DCE");
+    mir->spewPass("DCE");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("DCE")) {
@@ -1416,7 +1396,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!ReorderInstructions(mir, graph)) {
       return false;
     }
-    gs.spewPass("Reordering");
+    mir->spewPass("Reordering");
 
     AssertExtendedGraphCoherency(graph);
 
@@ -1431,7 +1411,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!MakeLoopsContiguous(graph)) {
       return false;
     }
-    gs.spewPass("Make loops contiguous");
+    mir->spewPass("Make loops contiguous");
     AssertExtendedGraphCoherency(graph);
 
     if (mir->shouldCancel("Make loops contiguous")) {
@@ -1448,7 +1428,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
       return false;
     }
 
-    gs.spewPass("Unroll loops");
+    mir->spewPass("Unroll loops");
 
     AssertExtendedGraphCoherency(graph);
 
@@ -1489,7 +1469,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
   // dominator tree.
   if (!mir->compilingWasm() && graph.osrBlock()) {
     graph.removeFakeLoopPredecessors();
-    gs.spewPass("Remove fake loop predecessors");
+    mir->spewPass("Remove fake loop predecessors");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Remove fake loop predecessors")) {
@@ -1505,7 +1485,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!edgeCaseAnalysis.analyzeLate()) {
       return false;
     }
-    gs.spewPass("Edge Case Analysis (Late)");
+    mir->spewPass("Edge Case Analysis (Late)");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Edge Case Analysis (Late)")) {
@@ -1521,7 +1501,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateRedundantChecks(graph)) {
       return false;
     }
-    gs.spewPass("Bounds Check Elimination");
+    mir->spewPass("Bounds Check Elimination");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Bounds Check Elimination")) {
@@ -1533,7 +1513,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateRedundantShapeGuards(graph)) {
       return false;
     }
-    gs.spewPass("Shape Guard Elimination");
+    mir->spewPass("Shape Guard Elimination");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Shape Guard Elimination")) {
@@ -1548,7 +1528,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!EliminateRedundantGCBarriers(graph)) {
       return false;
     }
-    gs.spewPass("GC Barrier Elimination");
+    mir->spewPass("GC Barrier Elimination");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("GC Barrier Elimination")) {
@@ -1560,7 +1540,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!FoldLoadsWithUnbox(mir, graph)) {
       return false;
     }
-    gs.spewPass("FoldLoadsWithUnbox");
+    mir->spewPass("FoldLoadsWithUnbox");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("FoldLoadsWithUnbox")) {
@@ -1572,7 +1552,7 @@ bool OptimizeMIR(MIRGenerator* mir) {
     if (!AddKeepAliveInstructions(graph)) {
       return false;
     }
-    gs.spewPass("Add KeepAlive Instructions");
+    mir->spewPass("Add KeepAlive Instructions");
     AssertGraphCoherency(graph);
 
     if (mir->shouldCancel("Add KeepAlive Instructions")) {
@@ -1593,7 +1573,6 @@ bool OptimizeMIR(MIRGenerator* mir) {
 
 LIRGraph* GenerateLIR(MIRGenerator* mir) {
   MIRGraph& graph = mir->graph();
-  GraphSpewer& gs = mir->graphSpewer();
 
   LIRGraph* lir = mir->alloc().lifoAlloc()->new_<LIRGraph>(&graph);
   if (!lir || !lir->init()) {
@@ -1605,7 +1584,7 @@ LIRGraph* GenerateLIR(MIRGenerator* mir) {
     if (!lirgen.generate()) {
       return nullptr;
     }
-    gs.spewPass("Generate LIR");
+    mir->spewPass("Generate LIR");
 
     if (mir->shouldCancel("Generate LIR")) {
       return nullptr;
@@ -1628,7 +1607,7 @@ LIRGraph* GenerateLIR(MIRGenerator* mir) {
       if (!regalloc.go()) {
         return nullptr;
       }
-      gs.spewPass("Allocate Registers [Backtracking]");
+      mir->spewPass("Allocate Registers [Backtracking]", &regalloc);
       break;
     }
     case RegisterAllocator_Simple: {
@@ -1636,7 +1615,7 @@ LIRGraph* GenerateLIR(MIRGenerator* mir) {
       if (!regalloc.go()) {
         return nullptr;
       }
-      gs.spewPass("Allocate Registers [Simple]");
+      mir->spewPass("Allocate Registers [Simple]");
       break;
     }
     default:
@@ -1708,7 +1687,7 @@ static AbortReasonOr<WarpSnapshot*> CreateWarpSnapshot(JSContext* cx,
   // Suppress GC during compilation.
   gc::AutoSuppressGC suppressGC(cx);
 
-  SpewBeginFunction(mirGen, script);
+  mirGen->spewBeginFunction(script);
 
   WarpOracle oracle(cx, *mirGen, script);
 
@@ -1841,7 +1820,7 @@ static AbortReason IonCompile(JSContext* cx, HandleScript script,
     AutoLockHelperThreadState lock;
     if (!StartOffThreadIonCompile(task, lock)) {
       JitSpew(JitSpew_IonAbort, "Unable to start off-thread ion compilation.");
-      mirGen->graphSpewer().endFunction();
+      mirGen->spewEndFunction();
       return AbortReason::Alloc;
     }
 
@@ -1880,55 +1859,24 @@ static AbortReason IonCompile(JSContext* cx, HandleScript script,
   return AbortReason::Disable;
 }
 
-static bool CheckFrame(JSContext* cx, BaselineFrame* frame) {
-  MOZ_ASSERT(!frame->isDebuggerEvalFrame());
+static void AssertBaselineFrameCanEnterIon(JSContext* cx,
+                                           BaselineFrame* frame) {
+  MOZ_ASSERT(jit::IsIonEnabled(cx));
   MOZ_ASSERT(!frame->isEvalFrame());
+  MOZ_ASSERT(frame->script()->canIonCompile());
+  MOZ_ASSERT(!frame->script()->isIonCompilingOffThread());
 
-  // This check is to not overrun the stack.
-  if (frame->isFunctionFrame()) {
-    if (TooManyActualArguments(frame->numActualArgs())) {
-      JitSpew(JitSpew_IonAbort, "too many actual arguments");
-      return false;
-    }
+  // Baseline has the same limit for the number of actual arguments, so if we
+  // entered Baseline we can also enter Ion.
+  MOZ_ASSERT_IF(frame->isFunctionFrame(),
+                !TooManyActualArguments(frame->numActualArgs()));
 
-    if (TooManyFormalArguments(frame->numFormalArgs())) {
-      JitSpew(JitSpew_IonAbort, "too many arguments");
-      return false;
-    }
-  }
-
-  return true;
+  // The number of formal arguments is checked in CanIonCompileScript. The
+  // Baseline JIT shouldn't attempt to tier up if that returns false.
+  MOZ_ASSERT_IF(frame->isFunctionFrame(),
+                !TooManyFormalArguments(frame->numFormalArgs()));
 }
 
-static bool CanIonCompileOrInlineScript(JSScript* script, const char** reason) {
-  if (script->isForEval()) {
-    // Eval frames are not yet supported. Supporting this will require new
-    // logic in pushBailoutFrame to deal with linking prev.
-    // Additionally, JSOp::GlobalOrEvalDeclInstantiation support will require
-    // baking in isEvalFrame().
-    *reason = "eval script";
-    return false;
-  }
-
-  if (script->isAsync()) {
-    if (script->isModule()) {
-      *reason = "async module";
-      return false;
-    }
-  }
-
-  if (script->hasNonSyntacticScope() && !script->function()) {
-    // Support functions with a non-syntactic global scope but not other
-    // scripts. For global scripts, WarpBuilder currently uses the global
-    // object as scope chain, this is not valid when the script has a
-    // non-syntactic global scope.
-    *reason = "has non-syntactic global scope";
-    return false;
-  }
-
-  return true;
-}  // namespace jit
-
 static bool ScriptIsTooLarge(JSContext* cx, JSScript* script) {
   if (!JitOptions.limitScriptSize) {
     return false;
@@ -1960,13 +1908,42 @@ bool CanIonCompileScript(JSContext* cx, JSScript* script) {
     return false;
   }
 
-  const char* reason = nullptr;
-  if (!CanIonCompileOrInlineScript(script, &reason)) {
-    JitSpew(JitSpew_IonAbort, "%s", reason);
+  if (script->isForEval()) {
+    // Eval frames are not yet supported. Fixing this will require adding
+    // support for the eval frame's environment chain, also for bailouts.
+    // Additionally, JSOp::GlobalOrEvalDeclInstantiation in WarpBuilder
+    // currently doesn't support eval scripts. See bug 1996190.
+    JitSpew(JitSpew_IonAbort, "eval script");
+    script->disableIon();
+    return false;
+  }
+
+  if (script->isAsync() && script->isModule()) {
+    // Async modules are not supported (bug 1996189).
+    JitSpew(JitSpew_IonAbort, "async module");
+    script->disableIon();
+    return false;
+  }
+
+  if (script->hasNonSyntacticScope() && !script->function()) {
+    // Support functions with a non-syntactic global scope but not other
+    // scripts. For global scripts, WarpBuilder currently uses the global
+    // object as scope chain, and this is not valid when the script has a
+    // non-syntactic global scope.
+    JitSpew(JitSpew_IonAbort, "has non-syntactic global scope");
+    script->disableIon();
+    return false;
+  }
+
+  if (script->function() &&
+      TooManyFormalArguments(script->function()->nargs())) {
+    JitSpew(JitSpew_IonAbort, "too many formal arguments");
+    script->disableIon();
     return false;
   }
 
   if (ScriptIsTooLarge(cx, script)) {
+    script->disableIon();
     return false;
   }
 
@@ -2073,13 +2050,6 @@ MethodStatus jit::CanEnterIon(JSContext* cx, RunState& state) {
       ForbidCompilation(cx, script);
       return Method_CantCompile;
     }
-
-    if (TooManyFormalArguments(
-            invoke.args().callee().as<JSFunction>().nargs())) {
-      JitSpew(JitSpew_IonAbort, "too many args");
-      ForbidCompilation(cx, script);
-      return Method_CantCompile;
-    }
   }
 
   // If --ion-eager is used, compile with Baseline first, so that we
@@ -2125,18 +2095,10 @@ MethodStatus jit::CanEnterIon(JSContext* cx, RunState& state) {
 
 static MethodStatus BaselineCanEnterAtEntry(JSContext* cx, HandleScript script,
                                             BaselineFrame* frame) {
-  MOZ_ASSERT(jit::IsIonEnabled(cx));
-  MOZ_ASSERT(script->canIonCompile());
-  MOZ_ASSERT(!script->isIonCompilingOffThread());
+  AssertBaselineFrameCanEnterIon(cx, frame);
   MOZ_ASSERT(!script->hasIonScript());
   MOZ_ASSERT(frame->isFunctionFrame());
 
-  // Mark as forbidden if frame can't be handled.
-  if (!CheckFrame(cx, frame)) {
-    ForbidCompilation(cx, script);
-    return Method_CantCompile;
-  }
-
   if (script->baselineScript()->hasPendingIonCompileTask()) {
     LinkIonScript(cx, script);
     if (script->hasIonScript()) {
@@ -2161,30 +2123,14 @@ static MethodStatus BaselineCanEnterAtEntry(JSContext* cx, HandleScript script,
 static MethodStatus BaselineCanEnterAtBranch(JSContext* cx, HandleScript script,
                                              BaselineFrame* osrFrame,
                                              jsbytecode* pc) {
-  MOZ_ASSERT(jit::IsIonEnabled(cx));
+  AssertBaselineFrameCanEnterIon(cx, osrFrame);
   MOZ_ASSERT((JSOp)*pc == JSOp::LoopHead);
 
-  // Skip if the script has been disabled.
-  if (!script->canIonCompile()) {
-    return Method_Skipped;
-  }
-
-  // Skip if the script is being compiled off thread.
-  if (script->isIonCompilingOffThread()) {
-    return Method_Skipped;
-  }
-
   // Optionally ignore on user request.
   if (!JitOptions.osr) {
     return Method_Skipped;
   }
 
-  // Mark as forbidden if frame can't be handled.
-  if (!CheckFrame(cx, osrFrame)) {
-    ForbidCompilation(cx, script);
-    return Method_CantCompile;
-  }
-
   // Check if the jitcode still needs to get linked and do this
   // to have a valid IonScript.
   if (script->baselineScript()->hasPendingIonCompileTask()) {
@@ -2463,10 +2409,6 @@ static void InvalidateActivation(JS::GCContext* gcx,
                 "#%zu baseline interpreter entry frame @ %p", frameno,
                 frame.fp());
         break;
-      case FrameType::Rectifier:
-        JitSpew(JitSpew_IonInvalidate, "#%zu rectifier frame @ %p", frameno,
-                frame.fp());
-        break;
       case FrameType::TrampolineNative:
         JitSpew(JitSpew_IonInvalidate, "#%zu TrampolineNative frame @ %p",
                 frameno, frame.fp());
