@@ -42,6 +42,7 @@
 #include "vm/BytecodeIterator.h"  // for AllBytecodesIterable
 #include "vm/BytecodeLocation.h"
 #include "vm/CodeCoverage.h"
+#include "vm/ConstantCompareOperand.h"
 #include "vm/EnvironmentObject.h"
 #include "vm/FrameIter.h"    // js::{,Script}FrameIter
 #include "vm/JSAtomUtils.h"  // AtomToPrintableString, Atomize
@@ -1649,6 +1650,7 @@ struct ExpressionDecompiler {
   bool quote(JSString* s, char quote);
   bool write(const char* s);
   bool write(JSString* str);
+  bool write(ConstantCompareOperand* operand);
   UniqueChars getOutput();
 #if defined(DEBUG) || defined(JS_JITSPEW)
   void setStackDump() { isStackDump = true; }
@@ -1901,6 +1903,14 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
              write(JSTypeToString(type)) && write("\")");
     }
 
+    case JSOp::StrictConstantEq:
+    case JSOp::StrictConstantNe: {
+      auto operand = ConstantCompareOperand::fromRawValue(GET_UINT16(pc));
+      return write("(") && decompilePCForStackOperand(pc, -1) && write(" ") &&
+             write(op == JSOp::StrictConstantEq ? "===" : "!==") &&
+             write(" ") && write(&operand) && write(")");
+    }
+
     case JSOp::InitElemArray:
       return write("[...]");
 
@@ -2216,6 +2226,25 @@ JSAtom* ExpressionDecompiler::loadAtom(jsbytecode* pc) {
   return script->getAtom(pc);
 }
 
+bool ExpressionDecompiler::write(ConstantCompareOperand* operand) {
+  switch (operand->type()) {
+    case ConstantCompareOperand::EncodedType::Int32: {
+      sprinter.printf("%d", operand->toInt32());
+      return true;
+    }
+    case ConstantCompareOperand::EncodedType::Boolean: {
+      return write(operand->toBoolean() ? "true" : "false");
+    }
+    case ConstantCompareOperand::EncodedType::Null: {
+      return write("null");
+    }
+    case ConstantCompareOperand::EncodedType::Undefined: {
+      return write("undefined");
+    }
+  }
+  MOZ_CRASH("Unknown constant compare operand type");
+}
+
 JSString* ExpressionDecompiler::loadString(jsbytecode* pc) {
   return script->getString(pc);
 }
@@ -2740,8 +2769,8 @@ static bool GetPCCountJSON(JSContext* cx, const ScriptAndCounts& sac,
   json.beginListProperty("opcodes");
 
   uint64_t hits = 0;
-  for (BytecodeRangeWithPosition range(cx, script); !range.empty();
-       range.popFront()) {
+  for (BytecodeRangeWithPosition range(cx, script, SkipPrologueOps::Yes);
+       !range.empty(); range.popFront()) {
     jsbytecode* pc = range.frontPC();
     size_t offset = script->pcToOffset(pc);
     JSOp op = JSOp(*pc);
