@@ -13,7 +13,6 @@
 
 #include "mozilla/URLPreloader.h"
 
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/Components.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/FileUtils.h"
@@ -25,9 +24,7 @@
 #include "mozilla/TaskController.h"
 #include "mozilla/glean/JsXpconnectMetrics.h"
 #include "mozilla/glean/XpcomMetrics.h"
-#include "mozilla/Telemetry.h"
 #include "mozilla/Try.h"
-#include "mozilla/Unused.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/ContentParent.h"
 #include "mozilla/dom/Document.h"
@@ -123,11 +120,12 @@ StaticAutoPtr<AutoMemMap> ScriptPreloader::gChildCacheData;
 
 ScriptPreloader& ScriptPreloader::GetSingleton() {
   if (!gScriptPreloader) {
+    AssertIsOnMainThread();
     if (XRE_IsParentProcess()) {
       gCacheData = new AutoMemMap();
       gScriptPreloader = new ScriptPreloader(gCacheData.get());
       gScriptPreloader->mChildCache = &GetChildSingleton();
-      Unused << gScriptPreloader->InitCache();
+      (void)gScriptPreloader->InitCache();
     } else {
       gScriptPreloader = &GetChildSingleton();
     }
@@ -161,10 +159,11 @@ ScriptPreloader& ScriptPreloader::GetSingleton() {
 //  previous cache file, but I'd rather do that as a follow-up.
 ScriptPreloader& ScriptPreloader::GetChildSingleton() {
   if (!gChildScriptPreloader) {
+    AssertIsOnMainThread();
     gChildCacheData = new AutoMemMap();
     gChildScriptPreloader = new ScriptPreloader(gChildCacheData.get());
     if (XRE_IsParentProcess()) {
-      Unused << gChildScriptPreloader->InitCache(u"scriptCache-child"_ns);
+      (void)gChildScriptPreloader->InitCache(u"scriptCache-child"_ns);
     }
   }
 
@@ -187,8 +186,10 @@ void ScriptPreloader::DeleteCacheDataSingleton() {
 }
 
 void ScriptPreloader::InitContentChild(ContentParent& parent) {
+  AssertIsOnMainThread();
+
   auto& cache = GetChildSingleton();
-  cache.mSaveMonitor.AssertOnWritingThread();
+  cache.mSaveMonitor.NoteOnMainThread();
 
   // We want startup script data from the first process of a given type.
   // That process sends back its script data before it executes any
@@ -212,10 +213,10 @@ void ScriptPreloader::InitContentChild(ContentParent& parent) {
   // Don't send original cache data to new processes if the cache has been
   // invalidated.
   if (fd.IsValid() && !cache.mCacheInvalidated) {
-    Unused << parent.SendPScriptCacheConstructor(fd, wantScriptData);
+    (void)parent.SendPScriptCacheConstructor(fd, wantScriptData);
   } else {
-    Unused << parent.SendPScriptCacheConstructor(NS_ERROR_FILE_NOT_FOUND,
-                                                 wantScriptData);
+    (void)parent.SendPScriptCacheConstructor(NS_ERROR_FILE_NOT_FOUND,
+                                             wantScriptData);
   }
 }
 
@@ -232,7 +233,7 @@ ProcessType ScriptPreloader::GetChildProcessType(const nsACString& remoteType) {
 ScriptPreloader::ScriptPreloader(AutoMemMap* cacheData)
     : mCacheData(cacheData),
       mMonitor("[ScriptPreloader.mMonitor]"),
-      mSaveMonitor("[ScriptPreloader.mSaveMonitor]", this) {
+      mSaveMonitor("[ScriptPreloader.mSaveMonitor]") {
   // We do not set the process type for child processes here because the
   // remoteType in ContentChild is not ready yet.
   if (XRE_IsParentProcess()) {
@@ -263,7 +264,7 @@ void ScriptPreloader::Cleanup() {
 void ScriptPreloader::StartCacheWrite() {
   MOZ_DIAGNOSTIC_ASSERT(!mSaveThread);
 
-  Unused << NS_NewNamedThread("SaveScripts", getter_AddRefs(mSaveThread), this);
+  (void)NS_NewNamedThread("SaveScripts", getter_AddRefs(mSaveThread), this);
 
   nsCOMPtr<nsIAsyncShutdownClient> barrier = GetShutdownBarrier();
   barrier->AddBlocker(this, NS_LITERAL_STRING_FROM_CSTRING(__FILE__), __LINE__,
@@ -300,18 +301,21 @@ void ScriptPreloader::InvalidateCache() {
   }
 
   {
-    MonitorSingleWriterAutoLock saveMonitorAutoLock(mSaveMonitor);
+    MonitorAutoLock saveMonitorAutoLock(mSaveMonitor.Lock());
+    mSaveMonitor.NoteExclusiveAccess();
 
     mCacheInvalidated = true;
   }
 
   // If we're waiting on a timeout to finish saving, interrupt it and just save
   // immediately.
-  mSaveMonitor.NotifyAll();
+  mSaveMonitor.Lock().NotifyAll();
 }
 
 nsresult ScriptPreloader::Observe(nsISupports* subject, const char* topic,
                                   const char16_t* data) {
+  AssertIsOnMainThread();
+
   nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
   if (!strcmp(topic, STARTUP_COMPLETE_TOPIC)) {
     obs->RemoveObserver(this, STARTUP_COMPLETE_TOPIC);
@@ -404,7 +408,7 @@ Result<nsCOMPtr<nsIFile>, nsresult> ScriptPreloader::GetCacheFile(
   MOZ_TRY(mProfD->Clone(getter_AddRefs(cacheFile)));
 
   MOZ_TRY(cacheFile->AppendNative("startupCache"_ns));
-  Unused << cacheFile->Create(nsIFile::DIRECTORY_TYPE, 0777);
+  (void)cacheFile->Create(nsIFile::DIRECTORY_TYPE, 0777);
 
   MOZ_TRY(cacheFile->Append(mBaseName + suffix));
 
@@ -420,8 +424,7 @@ Result<Ok, nsresult> ScriptPreloader::OpenCache() {
 
   MOZ_TRY(NS_GetSpecialDirectory("ProfLDS", getter_AddRefs(mProfD)));
 
-  nsCOMPtr<nsIFile> cacheFile;
-  MOZ_TRY_VAR(cacheFile, GetCacheFile(u".bin"_ns));
+  nsCOMPtr<nsIFile> cacheFile = MOZ_TRY(GetCacheFile(u".bin"_ns));
 
   bool exists;
   MOZ_TRY(cacheFile->Exists(&exists));
@@ -443,7 +446,6 @@ Result<Ok, nsresult> ScriptPreloader::OpenCache() {
 // Opens the script cache file for this session, and initializes the script
 // cache based on its contents. See WriteCache for details of the cache file.
 Result<Ok, nsresult> ScriptPreloader::InitCache(const nsAString& basePath) {
-  mSaveMonitor.AssertOnWritingThread();
   mCacheInitialized = true;
   mBaseName = basePath;
 
@@ -469,7 +471,6 @@ Result<Ok, nsresult> ScriptPreloader::InitCache(const nsAString& basePath) {
 
 Result<Ok, nsresult> ScriptPreloader::InitCache(
     const Maybe<ipc::FileDescriptor>& cacheFile, ScriptCacheChild* cacheChild) {
-  mSaveMonitor.AssertOnWritingThread();
   MOZ_ASSERT(XRE_IsContentProcess());
 
   mCacheInitialized = true;
@@ -629,8 +630,11 @@ void ScriptPreloader::PrepareCacheWriteInternal() {
     return;
   }
 
-  AutoSafeJSAPI jsapi;
-  JSAutoRealm ar(jsapi.cx(), xpc::PrivilegedJunkScope());
+  JS::FrontendContext* fc = JS::NewFrontendContext();
+  if (!fc) {
+    return;
+  }
+
   bool found = false;
   for (auto& script : IterHash(mScripts, Match<ScriptStatus::Saved>())) {
     // Don't write any scripts that are also in the child cache. They'll be
@@ -650,11 +654,13 @@ void ScriptPreloader::PrepareCacheWriteInternal() {
       found = true;
     }
 
-    if (!script->mSize && !script->XDREncode(jsapi.cx())) {
+    if (!script->mSize && !script->XDREncode(fc)) {
       script.Remove();
     }
   }
 
+  JS::DestroyFrontendContext(fc);
+
   if (!found) {
     mSaveComplete = true;
     return;
@@ -686,10 +692,9 @@ void ScriptPreloader::PrepareCacheWrite() {
 //   an offset from the start of the block, as specified above.
 Result<Ok, nsresult> ScriptPreloader::WriteCache() {
   MOZ_ASSERT(!NS_IsMainThread());
-  mSaveMonitor.AssertCurrentThreadOwns();
 
   if (!mDataPrepared && !mSaveComplete) {
-    MonitorSingleWriterAutoUnlock mau(mSaveMonitor);
+    MonitorAutoUnlock mau(mSaveMonitor.Lock());
 
     NS_DispatchAndSpinEventLoopUntilComplete(
         "ScriptPreloader::PrepareCacheWrite"_ns,
@@ -703,8 +708,7 @@ Result<Ok, nsresult> ScriptPreloader::WriteCache() {
     return Ok();
   }
 
-  nsCOMPtr<nsIFile> cacheFile;
-  MOZ_TRY_VAR(cacheFile, GetCacheFile(u"-new.bin"_ns));
+  nsCOMPtr<nsIFile> cacheFile = MOZ_TRY(GetCacheFile(u"-new.bin"_ns));
 
   bool exists;
   MOZ_TRY(cacheFile->Exists(&exists));
@@ -791,7 +795,8 @@ nsresult ScriptPreloader::GetName(nsACString& aName) {
 // Runs in the mSaveThread thread, and writes out the cache file for the next
 // session after a reasonable delay.
 nsresult ScriptPreloader::Run() {
-  MonitorSingleWriterAutoLock mal(mSaveMonitor);
+  MonitorAutoLock mal(mSaveMonitor.Lock());
+  mSaveMonitor.NoteLockHeld();
 
   // Ideally wait about 10 seconds before saving, to avoid unnecessary IO
   // during early startup. But only if the cache hasn't been invalidated,
@@ -802,16 +807,16 @@ nsresult ScriptPreloader::Run() {
   }
 
   auto result = URLPreloader::GetSingleton().WriteCache();
-  Unused << NS_WARN_IF(result.isErr());
+  (void)NS_WARN_IF(result.isErr());
 
   result = WriteCache();
-  Unused << NS_WARN_IF(result.isErr());
+  (void)NS_WARN_IF(result.isErr());
 
   {
-    MonitorSingleWriterAutoLock lock(mChildCache->mSaveMonitor);
+    MonitorAutoLock lock(mChildCache->mSaveMonitor.Lock());
     result = mChildCache->WriteCache();
   }
-  Unused << NS_WARN_IF(result.isErr());
+  (void)NS_WARN_IF(result.isErr());
 
   NS_DispatchToMainThread(
       NewRunnableMethod("ScriptPreloader::CacheWriteComplete", this,
@@ -956,16 +961,19 @@ already_AddRefed<JS::Stencil> ScriptPreloader::GetCachedStencil(
     RefPtr<JS::Stencil> stencil =
         mChildCache->GetCachedStencilInternal(cx, options, path);
     if (stencil) {
-      Telemetry::AccumulateCategorical(
-          Telemetry::LABELS_SCRIPT_PRELOADER_REQUESTS::HitChild);
+      glean::script_preloader::requests
+          .EnumGet(glean::script_preloader::RequestsLabel::eHitchild)
+          .Add();
       return stencil.forget();
     }
   }
 
   RefPtr<JS::Stencil> stencil = GetCachedStencilInternal(cx, options, path);
-  Telemetry::AccumulateCategorical(
-      stencil ? Telemetry::LABELS_SCRIPT_PRELOADER_REQUESTS::Hit
-              : Telemetry::LABELS_SCRIPT_PRELOADER_REQUESTS::Miss);
+  glean::script_preloader::requests
+      .EnumGet(stencil ? glean::script_preloader::RequestsLabel::eHit
+                       : glean::script_preloader::RequestsLabel::eMiss)
+      .Add();
+
   return stencil.forget();
 }
 
@@ -1021,10 +1029,9 @@ already_AddRefed<JS::Stencil> ScriptPreloader::WaitForCachedStencil(
           }
         }
 
-        double waitedMS = (TimeStamp::Now() - start).ToMilliseconds();
-        Telemetry::Accumulate(Telemetry::SCRIPT_PRELOADER_WAIT_TIME,
-                              int(waitedMS));
-        LOG(Debug, "Waited %fms\n", waitedMS);
+        TimeDuration waited = TimeStamp::Now() - start;
+        glean::script_preloader::wait_time.AccumulateRawDuration(waited);
+        LOG(Debug, "Waited %fms\n", waited.ToMilliseconds());
       }
     }
   }
@@ -1265,19 +1272,20 @@ ScriptPreloader::CachedStencil::CachedStencil(ScriptPreloader& cache,
   mProcessTypes = {};
 }
 
-bool ScriptPreloader::CachedStencil::XDREncode(JSContext* cx) {
+bool ScriptPreloader::CachedStencil::XDREncode(JS::FrontendContext* aFc) {
   auto cleanup = MakeScopeExit([&]() { MaybeDropStencil(); });
 
   mXDRData.construct<JS::TranscodeBuffer>();
 
-  JS::TranscodeResult code = JS::EncodeStencil(cx, mStencil, Buffer());
+  JS::TranscodeResult code = JS::EncodeStencil(aFc, mStencil, Buffer());
+
   if (code == JS::TranscodeResult::Ok) {
     mXDRRange.emplace(Buffer().begin(), Buffer().length());
     mSize = Range().length();
     return true;
   }
   mXDRData.destroy();
-  JS_ClearPendingException(cx);
+  JS::ClearFrontendErrors(aFc);
   return false;
 }
 
@@ -1352,7 +1360,7 @@ nsresult ScriptPreloader::BlockShutdown(
     nsIAsyncShutdownClient* aBarrierClient) {
   // If we're waiting on a timeout to finish saving, interrupt it and just save
   // immediately.
-  mSaveMonitor.NotifyAll();
+  mSaveMonitor.Lock().NotifyAll();
   return NS_OK;
 }
 
@@ -1361,7 +1369,7 @@ already_AddRefed<nsIAsyncShutdownClient> ScriptPreloader::GetShutdownBarrier() {
   MOZ_RELEASE_ASSERT(svc);
 
   nsCOMPtr<nsIAsyncShutdownClient> barrier;
-  Unused << svc->GetXpcomWillShutdown(getter_AddRefs(barrier));
+  (void)svc->GetXpcomWillShutdown(getter_AddRefs(barrier));
   MOZ_RELEASE_ASSERT(barrier);
 
   return barrier.forget();
