@@ -36,7 +36,6 @@
 #include "DNSAdditionalInfo.h"
 #include "TRRService.h"
 
-#include "mozilla/Attributes.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/net/NeckoCommon.h"
 #include "mozilla/net/ChildDNSService.h"
@@ -46,7 +45,6 @@
 #include "mozilla/StaticPtr.h"
 #include "mozilla/SyncRunnable.h"
 #include "mozilla/TextUtils.h"
-#include "mozilla/Utf8.h"
 
 using namespace mozilla;
 using namespace mozilla::net;
@@ -364,6 +362,12 @@ NS_IMETHODIMP nsDNSRecord::GetTrrSkipReason(
 NS_IMETHODIMP
 nsDNSRecord::GetTtl(uint32_t* aTtl) { return mHostRecord->GetTtl(aTtl); }
 
+NS_IMETHODIMP
+nsDNSRecord::GetLastUpdate(mozilla::TimeStamp* aLastUpdate) {
+  MutexAutoLock lock(mHostRecord->addr_info_lock);
+  return mHostRecord->GetLastUpdate(aLastUpdate);
+}
+
 class nsDNSByTypeRecord : public nsIDNSByTypeRecord,
                           public nsIDNSTXTRecord,
                           public nsIDNSHTTPSSVCRecord {
@@ -566,18 +570,53 @@ nsDNSAsyncRequest::Cancel(nsresult reason) {
 
 //-----------------------------------------------------------------------------
 
-class nsDNSSyncRequest : public nsResolveHostCallback {
+class DNSCacheRequest : public nsResolveHostCallback {
+ public:
   NS_DECL_THREADSAFE_ISUPPORTS
+
+  DNSCacheRequest() = default;
+
+  void OnResolveHostComplete(nsHostResolver* resolver, nsHostRecord* hostRecord,
+                             nsresult status) override {
+    mStatus = status;
+    mHostRecord = hostRecord;
+  }
+
+  bool EqualsAsyncListener(nsIDNSListener* aListener) override {
+    // Sync request: no listener to compare
+    return false;
+  }
+
+  size_t SizeOfIncludingThis(
+      mozilla::MallocSizeOf mallocSizeOf) const override {
+    size_t n = mallocSizeOf(this);
+
+    // The following fields aren't measured.
+    // - mHostRecord, because it's a non-owning pointer
+
+    // Measurement of the following members may be added later if DMD finds it
+    // is worthwhile:
+    // - nsDNSSyncRequest::mMonitor
+
+    return n;
+  }
+
+  nsresult mStatus = NS_OK;
+  RefPtr<nsHostRecord> mHostRecord;
+
+ protected:
+  virtual ~DNSCacheRequest() = default;
+};
+
+NS_IMPL_ISUPPORTS0(DNSCacheRequest)
+
+class nsDNSSyncRequest : public DNSCacheRequest {
  public:
   explicit nsDNSSyncRequest(PRMonitor* mon) : mMonitor(mon) {}
 
   void OnResolveHostComplete(nsHostResolver*, nsHostRecord*, nsresult) override;
-  bool EqualsAsyncListener(nsIDNSListener* aListener) override;
-  size_t SizeOfIncludingThis(mozilla::MallocSizeOf) const override;
 
   bool mDone = false;
-  nsresult mStatus = NS_OK;
-  RefPtr<nsHostRecord> mHostRecord;
 
  private:
   virtual ~nsDNSSyncRequest() = default;
@@ -585,38 +624,17 @@ class nsDNSSyncRequest : public nsResolveHostCallback {
   PRMonitor* mMonitor = nullptr;
 };
 
-NS_IMPL_ISUPPORTS0(nsDNSSyncRequest)
-
 void nsDNSSyncRequest::OnResolveHostComplete(nsHostResolver* resolver,
                                              nsHostRecord* hostRecord,
                                              nsresult status) {
   // store results, and wake up nsDNSService::Resolve to process results.
   PR_EnterMonitor(mMonitor);
   mDone = true;
-  mStatus = status;
-  mHostRecord = hostRecord;
+  DNSCacheRequest::OnResolveHostComplete(resolver, hostRecord, status);
   PR_Notify(mMonitor);
   PR_ExitMonitor(mMonitor);
 }
 
-bool nsDNSSyncRequest::EqualsAsyncListener(nsIDNSListener* aListener) {
-  // Sync request: no listener to compare
-  return false;
-}
-
-size_t nsDNSSyncRequest::SizeOfIncludingThis(MallocSizeOf mallocSizeOf) const {
-  size_t n = mallocSizeOf(this);
-
-  // The following fields aren't measured.
-  // - mHostRecord, because it's a non-owning pointer
-
-  // Measurement of the following members may be added later if DMD finds it
-  // is worthwhile:
-  // - mMonitor
-
-  return n;
-}
-
 class NotifyDNSResolution : public Runnable {
  public:
   explicit NotifyDNSResolution(const nsACString& aHostname)
@@ -1185,8 +1203,10 @@ nsDNSService::ResolveNative(const nsACString& aHostname,
                             nsIDNSService::DNSFlags flags,
                             const OriginAttributes& aOriginAttributes,
                             nsIDNSRecord** result) {
-  // Synchronous resolution is not available on the main thread.
-  if (NS_IsMainThread()) {
+  // Synchronous resolution is not allowed on the main thread.
+  // However, if RESOLVE_OFFLINE is set, we're only reading from the DNS cache,
+  // so it's safe to allow this on the main thread.
+  if (NS_IsMainThread() && !(flags & nsIDNSService::RESOLVE_OFFLINE)) {
     return NS_ERROR_NOT_AVAILABLE;
   }
 
@@ -1233,6 +1253,20 @@ nsresult nsDNSService::ResolveInternal(
     return NS_ERROR_UNKNOWN_PROXY_HOST;
   }
 
+  //  Since RESOLVE_OFFLINE is set, we can use DNSCacheRequest to retrieve the
+  //  cached result directly.
+  if (flags & RESOLVE_OFFLINE) {
+    RefPtr<DNSCacheRequest> req = new DNSCacheRequest();
+    uint16_t af = GetAFForLookup(hostname, flags);
+    rv = res->ResolveHost(hostname, ""_ns, -1, RESOLVE_TYPE_DEFAULT,
+                          aOriginAttributes, flags, af, req);
+    if (NS_SUCCEEDED(rv)) {
+      RefPtr<nsDNSRecord> rec = new nsDNSRecord(req->mHostRecord);
+      rec.forget(result);
+    }
+    return rv;
+  }
+
   //
   // sync resolve: since the host resolver only works asynchronously, we need
   // to use a mutex and a condvar to wait for the result.  however, since the
@@ -1394,7 +1428,7 @@ NS_IMETHODIMP
 nsDNSService::ClearCache(bool aTrrToo) {
   RefPtr<nsHostResolver> resolver = GetResolverLocked();
   NS_ENSURE_TRUE(resolver, NS_ERROR_NOT_INITIALIZED);
-  resolver->FlushCache(aTrrToo);
+  resolver->FlushCache(aTrrToo, true);
   return NS_OK;
 }
 
