@@ -100,7 +100,7 @@ nsresult HttpTransactionChild::InitInternal(
     return rv;
   }
 
-  Unused << mTransaction->AsyncRead(this, getter_AddRefs(mTransactionPump));
+  (void)mTransaction->AsyncRead(this, getter_AddRefs(mTransactionPump));
   return rv;
 }
 
@@ -464,7 +464,7 @@ HttpTransactionChild::OnStartRequest(nsIRequest* aRequest) {
   HttpConnectionInfoCloneArgs infoArgs;
   nsHttpConnectionInfo::SerializeHttpConnectionInfo(connInfo, infoArgs);
 
-  Unused << SendOnStartRequest(
+  (void)SendOnStartRequest(
       status, std::move(optionalHead), securityInfo,
       mTransaction->ProxyConnectFailed(),
       ToTimingStructArgs(mTransaction->Timings()), proxyConnectResponseCode,
@@ -546,11 +546,11 @@ HttpTransactionChild::OnStopRequest(nsIRequest* aRequest, nsresult aStatus) {
     mDataBridgeParent = nullptr;
   }
 
-  Unused << SendOnStopRequest(aStatus, mTransaction->ResponseIsComplete(),
-                              mTransaction->GetTransferSize(),
-                              ToTimingStructArgs(mTransaction->Timings()),
-                              responseTrailers, mTransactionObserverResult,
-                              lastActTabOpt, TimeStamp::Now());
+  (void)SendOnStopRequest(aStatus, mTransaction->ResponseIsComplete(),
+                          mTransaction->GetTransferSize(),
+                          ToTimingStructArgs(mTransaction->Timings()),
+                          responseTrailers, mTransactionObserverResult,
+                          lastActTabOpt, TimeStamp::Now());
 
   return NS_OK;
 }
@@ -598,7 +598,7 @@ HttpTransactionChild::OnTransportStatus(nsITransport* aTransport,
     arg.emplace(selfAddr, peerAddr, isTrr, mode, reason, echConfigUsed);
   }
 
-  Unused << SendOnTransportStatus(aStatus, aProgress, aProgressMax, arg);
+  (void)SendOnTransportStatus(aStatus, aProgress, aProgressMax, arg);
   return NS_OK;
 }
 
@@ -637,7 +637,7 @@ HttpTransactionChild::EarlyHint(const nsACString& aValue,
                                 const nsACString& aCSPHeader) {
   LOG(("HttpTransactionChild::EarlyHint"));
   if (CanSend()) {
-    Unused << SendEarlyHint(aValue, aReferrerPolicy, aCSPHeader);
+    (void)SendEarlyHint(aValue, aReferrerPolicy, aCSPHeader);
   }
   return NS_OK;
 }
