@@ -287,13 +287,9 @@ nsresult InterceptedHttpChannel::RedirectForResponseURL(
   nsCOMPtr<nsILoadInfo> redirectLoadInfo =
       CloneLoadInfoForRedirect(aResponseURI, flags);
 
-  ExtContentPolicyType contentPolicyType =
-      redirectLoadInfo->GetExternalContentPolicyType();
-
-  rv = newChannel->Init(aResponseURI, mCaps,
-                        static_cast<nsProxyInfo*>(mProxyInfo.get()),
-                        mProxyResolveFlags, mProxyURI, mChannelId,
-                        contentPolicyType, redirectLoadInfo);
+  rv = newChannel->Init(
+      aResponseURI, mCaps, static_cast<nsProxyInfo*>(mProxyInfo.get()),
+      mProxyResolveFlags, mProxyURI, mChannelId, redirectLoadInfo);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Normally we don't propagate the LoadInfo's service worker tainting
@@ -355,7 +351,7 @@ nsresult InterceptedHttpChannel::StartPump() {
   // TODO: We could implement an nsIFixedLengthInputStream interface and
   //       QI to it here.  This would let us determine the total length
   //       for streams that support it.  See bug 1388774.
-  Unused << GetContentLength(&mSynthesizedStreamLength);
+  (void)GetContentLength(&mSynthesizedStreamLength);
 
   nsresult rv =
       nsInputStreamPump::Create(getter_AddRefs(mPump), mBodyReader, 0, 0, true);
@@ -692,7 +688,7 @@ void InterceptedHttpChannel::DoNotifyListenerCleanup() {
 }
 
 void InterceptedHttpChannel::DoAsyncAbort(nsresult aStatus) {
-  Unused << AsyncAbort(aStatus);
+  (void)AsyncAbort(aStatus);
 }
 
 namespace {
@@ -944,7 +940,7 @@ InterceptedHttpChannel::StartSynthesizedResponse(
   }
 
   bool equal = false;
-  Unused << mURI->Equals(responseURI, &equal);
+  (void)mURI->Equals(responseURI, &equal);
   if (!equal) {
     rv = RedirectForResponseURL(responseURI, aResponseRedirected);
     NS_ENSURE_SUCCESS(rv, rv);
