@@ -47,6 +47,12 @@
 using namespace mozilla;
 using namespace mozilla::dom;
 
+// If you are adding fields to a tree op and this static assert fails,
+// please consider reordering the fields to avoid excess padding or,
+// if that doesn't help, splitting a rare operation into multiple
+// tree ops before allowing the size of all operations to get larger.
+static_assert(sizeof(nsHtml5TreeOperation) <= 56);
+
 /**
  * Helper class that opens a notification batch if the current doc
  * is different from the executor doc.
@@ -209,6 +215,8 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation() {
 
     void operator()(const opEnableEncodingMenu& aOperation) {}
 
+    void operator()(const opMicrotaskCheckpoint& aOperation) {}
+
     void operator()(const uninitialized& aOperation) {
       NS_WARNING("Uninitialized tree op.");
     }
@@ -224,7 +232,8 @@ nsresult nsHtml5TreeOperation::AppendTextToTextNode(
   MOZ_ASSERT(aBuilder);
   MOZ_ASSERT(aBuilder->IsInDocUpdate());
   uint32_t oldLength = aTextNode->TextLength();
-  CharacterDataChangeInfo info = {true, oldLength, oldLength, aLength};
+  CharacterDataChangeInfo info = {true, oldLength, oldLength, aLength,
+                                  MutationEffectOnScript::KeepTrustWorthiness};
   MutationObservers::NotifyCharacterDataWillChange(aTextNode, info);
 
   nsresult rv = aTextNode->AppendText(aBuffer, aLength, false);
@@ -264,7 +273,8 @@ nsresult nsHtml5TreeOperation::Append(nsIContent* aNode, nsIContent* aParent,
   aParent->AppendChildTo(aNode, false, rv);
   if (!rv.Failed() && !ownerDoc->DOMNotificationsSuspended()) {
     aNode->SetParserHasNotified();
-    MutationObservers::NotifyContentAppended(aParent, aNode);
+    MutationObservers::NotifyContentAppended(
+        aParent, aNode, {MutationEffectOnScript::KeepTrustWorthiness});
   }
   return rv.StealNSResult();
 }
@@ -308,7 +318,8 @@ nsresult nsHtml5TreeOperation::AppendToDocument(
 
   if (!doc->DOMNotificationsSuspended()) {
     aNode->SetParserHasNotified();
-    MutationObservers::NotifyContentInserted(doc, aNode);
+    MutationObservers::NotifyContentInserted(
+        doc, aNode, {MutationEffectOnScript::KeepTrustWorthiness});
   }
 
   NS_ASSERTION(!nsContentUtils::IsSafeToRunScript(),
@@ -343,7 +354,8 @@ void nsHtml5TreeOperation::Detach(nsIContent* aNode,
   nsCOMPtr<nsINode> parent = aNode->GetParentNode();
   if (parent) {
     nsHtml5OtherDocUpdate update(parent->OwnerDoc(), aBuilder->GetDocument());
-    parent->RemoveChildNode(aNode, true);
+    parent->RemoveChildNode(aNode, true, nullptr, nullptr,
+                            MutationEffectOnScript::KeepTrustWorthiness);
   }
 }
 
@@ -356,7 +368,8 @@ nsresult nsHtml5TreeOperation::AppendChildrenToNewParent(
   bool didAppend = false;
   while (aNode->HasChildren()) {
     nsCOMPtr<nsIContent> child = aNode->GetFirstChild();
-    aNode->RemoveChildNode(child, true);
+    aNode->RemoveChildNode(child, true, nullptr, nullptr,
+                           MutationEffectOnScript::KeepTrustWorthiness);
 
     ErrorResult rv;
     aParent->AppendChildTo(child, false, rv);
@@ -366,7 +379,9 @@ nsresult nsHtml5TreeOperation::AppendChildrenToNewParent(
     didAppend = true;
   }
   if (didAppend) {
-    MutationObservers::NotifyContentAppended(aParent, aParent->GetLastChild());
+    MutationObservers::NotifyContentAppended(
+        aParent, aParent->GetLastChild(),
+        {MutationEffectOnScript::KeepTrustWorthiness});
   }
   return NS_OK;
 }
@@ -388,7 +403,8 @@ nsresult nsHtml5TreeOperation::FosterParent(nsIContent* aNode,
       return rv.StealNSResult();
     }
 
-    MutationObservers::NotifyContentInserted(foster, aNode);
+    MutationObservers::NotifyContentInserted(
+        foster, aNode, {MutationEffectOnScript::KeepTrustWorthiness});
     return NS_OK;
   }
 
@@ -504,7 +520,9 @@ nsIContent* nsHtml5TreeOperation::CreateHTMLElement(
     AutoSetThrowOnDynamicMarkupInsertionCounter
         throwOnDynamicMarkupInsertionCounter(document);
     nsHtml5AutoPauseUpdate autoPauseContentUpdate(aBuilder);
-    { nsAutoMicroTask mt; }
+    {
+      nsAutoMicroTask mt;
+    }
     AutoCEReaction autoCEReaction(
         document->GetDocGroup()->CustomElementReactionsStack(), nullptr);
     return DoCreateElement(nullptr);
@@ -674,7 +692,8 @@ nsresult nsHtml5TreeOperation::FosterParentText(
       return error.StealNSResult();
     }
 
-    MutationObservers::NotifyContentInserted(foster, text);
+    MutationObservers::NotifyContentInserted(
+        foster, text, {MutationEffectOnScript::KeepTrustWorthiness});
     return rv;
   }
 
@@ -939,7 +958,8 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
           *aOperation.mHost, aOperation.mShadowRootMode,
           aOperation.mShadowRootIsClonable,
           aOperation.mShadowRootIsSerializable,
-          aOperation.mShadowRootDelegatesFocus);
+          aOperation.mShadowRootDelegatesFocus,
+          aOperation.mShadowRootReferenceTarget);
       if (root) {
         *aOperation.mFragHandle = root;
         return NS_OK;
@@ -1234,6 +1254,12 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       return NS_OK;
     }
 
+    nsresult operator()(const opMicrotaskCheckpoint& aOperation) {
+      nsHtml5AutoPauseUpdate autoPauseContentUpdate(mBuilder);
+      nsAutoMicroTask mt;
+      return NS_OK;
+    }
+
     nsresult operator()(const uninitialized& aOperation) {
       MOZ_CRASH("uninitialized");
       return NS_OK;
