@@ -42,9 +42,7 @@
 #include "mozilla/Range.h"
 #include "mozilla/SandboxInfo.h"
 #include "mozilla/StackWalk.h"
-#include "mozilla/Span.h"
 #include "mozilla/UniquePtr.h"
-#include "mozilla/Unused.h"
 #include "mozilla/ipc/UtilityProcessSandboxing.h"
 #include "prenv.h"
 #include "base/posix/eintr_wrapper.h"
@@ -189,7 +187,7 @@ static void InstallSigSysHandler(void) {
   struct sigaction act;
 
   // Ensure that the Chromium handler is installed.
-  Unused << sandbox::Trap::Registry();
+  (void)sandbox::Trap::Registry();
 
   // If the signal handling state isn't as expected, crash now instead
   // of crashing later (and more confusingly) when SIGSYS happens.
@@ -484,7 +482,7 @@ static void BroadcastSetThreadSandbox(const sock_fprog* aFilter) {
     MOZ_CRASH("handler for the signal was changed to another");
   }
   gSeccompTsyncBroadcastSignum = 0;
-  Unused << closedir(taskdp);
+  (void)closedir(taskdp);
   // And now, deprivilege the main thread:
   SetThreadSandbox();
   gSetSandboxFilter = nullptr;
@@ -607,7 +605,7 @@ static void SandboxLateInit() {
   // This will create:
   //  - pointers to uprofiler to make use of the profiler
   //  - a SandboxProfiler
-  //  - a MPSCQueue
+  //  - a BoundedMPSCQueue
   //  - a std::thread
   //
   // So that later usage of uprofiler under SIGSYS context can:
