@@ -13,6 +13,7 @@ use style_traits::ToCss;
 
 use crate::logical_geometry::PhysicalSide;
 use crate::values::animated::ToAnimatedZero;
+use crate::values::computed::position::TryTacticAdjustment;
 use crate::values::generics::box_::PositionProperty;
 use crate::values::generics::length::GenericAnchorSizeFunction;
 use crate::values::generics::ratio::Ratio;
@@ -36,6 +37,7 @@ use crate::values::DashedIdent;
     ToComputedValue,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C)]
 pub struct GenericPosition<H, V> {
@@ -96,6 +98,7 @@ pub trait PositionComponent {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C, u8)]
 pub enum GenericPositionOrAuto<Pos> {
@@ -138,6 +141,7 @@ impl<Pos> PositionOrAuto<Pos> {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C, u8)]
 pub enum GenericZIndex<I> {
@@ -217,6 +221,7 @@ pub enum PreferredRatio<N> {
     ToCss,
     ToResolvedValue,
     ToShmem,
+    ToTyped,
 )]
 #[repr(C)]
 pub struct GenericAspectRatio<N> {
@@ -272,6 +277,7 @@ impl<N> ToAnimatedZero for AspectRatio<N> {
     ToAnimatedZero,
     ToComputedValue,
     ToResolvedValue,
+    ToTyped,
 )]
 #[repr(C)]
 pub enum GenericInset<P, LP> {
@@ -282,19 +288,11 @@ pub enum GenericInset<P, LP> {
     /// Inset defined by the anchor element.
     ///
     /// <https://drafts.csswg.org/css-anchor-position-1/#anchor-pos>
-    AnchorFunction(
-        #[animation(field_bound)]
-        #[distance(field_bound)]
-        Box<GenericAnchorFunction<P, LP>>,
-    ),
+    AnchorFunction(Box<GenericAnchorFunction<P, Self>>),
     /// Inset defined by the size of the anchor element.
     ///
     /// <https://drafts.csswg.org/css-anchor-position-1/#anchor-pos>
-    AnchorSizeFunction(
-        #[animation(field_bound)]
-        #[distance(field_bound)]
-        Box<GenericAnchorSizeFunction<LP>>,
-    ),
+    AnchorSizeFunction(Box<GenericAnchorSizeFunction<Self>>),
     /// A `<length-percentage>` value, guaranteed to contain `calc()`,
     /// which then is guaranteed to contain `anchor()` or `anchor-size()`.
     AnchorContainingCalcFunction(LP),
@@ -351,7 +349,7 @@ pub use self::GenericInset as Inset;
     Deserialize,
 )]
 #[repr(C)]
-pub struct GenericAnchorFunction<Percentage, LengthPercentage> {
+pub struct GenericAnchorFunction<Percentage, Fallback> {
     /// Anchor name of the element to anchor to.
     /// If omitted, selects the implicit anchor element.
     #[animation(constant)]
@@ -360,13 +358,13 @@ pub struct GenericAnchorFunction<Percentage, LengthPercentage> {
     /// the anchored element to.
     pub side: GenericAnchorSide<Percentage>,
     /// Value to use in case the anchor function is invalid.
-    pub fallback: Optional<LengthPercentage>,
+    pub fallback: Optional<Fallback>,
 }
 
-impl<Percentage, LengthPercentage> ToCss for GenericAnchorFunction<Percentage, LengthPercentage>
+impl<Percentage, Fallback> ToCss for GenericAnchorFunction<Percentage, Fallback>
 where
     Percentage: ToCss,
-    LengthPercentage: ToCss,
+    Fallback: ToCss,
 {
     fn to_css<W>(&self, dest: &mut CssWriter<W>) -> std::fmt::Result
     where
@@ -387,7 +385,7 @@ where
     }
 }
 
-impl<Percentage, LengthPercentage> GenericAnchorFunction<Percentage, LengthPercentage> {
+impl<Percentage, Fallback> GenericAnchorFunction<Percentage, Fallback> {
     /// Is the anchor valid for given property?
     pub fn valid_for(&self, side: PhysicalSide, position_property: PositionProperty) -> bool {
         position_property.is_absolutely_positioned() && self.side.valid_for(side)
@@ -442,18 +440,72 @@ pub enum AnchorSideKeyword {
     Center,
 }
 
+impl AnchorSideKeyword {
+    fn from_physical_side(side: PhysicalSide) -> Self {
+        match side {
+            PhysicalSide::Top => Self::Top,
+            PhysicalSide::Right => Self::Right,
+            PhysicalSide::Bottom => Self::Bottom,
+            PhysicalSide::Left => Self::Left,
+        }
+    }
+
+    fn physical_side(self) -> Option<PhysicalSide> {
+        Some(match self {
+            Self::Top => PhysicalSide::Top,
+            Self::Right => PhysicalSide::Right,
+            Self::Bottom => PhysicalSide::Bottom,
+            Self::Left => PhysicalSide::Left,
+            _ => return None,
+        })
+    }
+}
+
+impl TryTacticAdjustment for AnchorSideKeyword {
+    fn try_tactic_adjustment(&mut self, old_side: PhysicalSide, new_side: PhysicalSide) {
+        if !old_side.parallel_to(new_side) {
+            let Some(s) = self.physical_side() else {
+                return;
+            };
+            *self = Self::from_physical_side(if s == new_side {
+                old_side
+            } else if s == old_side {
+                new_side
+            } else if s == new_side.opposite_side() {
+                old_side.opposite_side()
+            } else {
+                debug_assert_eq!(s, old_side.opposite_side());
+                new_side.opposite_side()
+            });
+            return;
+        }
+
+        *self = match self {
+            Self::Center | Self::Inside | Self::Outside => *self,
+            Self::SelfStart => Self::SelfEnd,
+            Self::SelfEnd => Self::SelfStart,
+            Self::Start => Self::End,
+            Self::End => Self::Start,
+            Self::Top => Self::Bottom,
+            Self::Bottom => Self::Top,
+            Self::Left => Self::Right,
+            Self::Right => Self::Left,
+        }
+    }
+}
+
 impl AnchorSideKeyword {
     fn valid_for(&self, side: PhysicalSide) -> bool {
         match self {
             Self::Left | Self::Right => matches!(side, PhysicalSide::Left | PhysicalSide::Right),
             Self::Top | Self::Bottom => matches!(side, PhysicalSide::Top | PhysicalSide::Bottom),
-            Self::Inside |
-            Self::Outside |
-            Self::Start |
-            Self::End |
-            Self::SelfStart |
-            Self::SelfEnd |
-            Self::Center => true,
+            Self::Inside
+            | Self::Outside
+            | Self::Start
+            | Self::End
+            | Self::SelfStart
+            | Self::SelfEnd
+            | Self::Center => true,
         }
     }
 }
