@@ -25,7 +25,6 @@
 #include "plbase64.h"
 #include "nsIClassInfoImpl.h"
 #include "mozilla/AppShutdown.h"
-#include "mozilla/ArrayUtils.h"
 #include "mozilla/LoadInfo.h"
 #include "mozilla/NullPrincipal.h"
 #include "mozilla/Preferences.h"
@@ -219,7 +218,7 @@ void nsFaviconService::ClearImageCache(nsIURI* aImageURI) {
       GetImgTools()->GetImgCacheForDocument(nullptr, getter_AddRefs(imgCache));
   MOZ_ASSERT(NS_SUCCEEDED(rv));
   if (NS_SUCCEEDED(rv)) {
-    Unused << imgCache->RemoveEntry(aImageURI, nullptr);
+    (void)imgCache->RemoveEntry(aImageURI, nullptr);
   }
 }
 
@@ -243,22 +242,19 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
     return result.StealNSResult();
   }
 
-  nsresult rv = NS_OK;
   auto guard = MakeScopeExit([&]() {
-    if (NS_SUCCEEDED(rv)) {
-      promise->MaybeResolveWithUndefined();
-      promise.forget(aPromise);
-    }
+    promise->MaybeResolveWithUndefined();
+    promise.forget(aPromise);
   });
 
   if (!aDataURL->SchemeIs("data")) {
-    return (rv = NS_ERROR_INVALID_ARG);
+    return NS_ERROR_INVALID_ARG;
   }
 
   NS_ENSURE_ARG(canStoreIconForPage(aPageURI));
 
   if (AppShutdown::IsInOrBeyond(ShutdownPhase::AppShutdownConfirmed)) {
-    return (rv = NS_OK);
+    return NS_OK;
   }
 
   PRTime now = PR_Now();
@@ -267,6 +263,7 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
     aExpiration = now + MAX_FAVICON_EXPIRATION;
   }
 
+  nsresult rv;
   nsCOMPtr<nsIIOService> ioService = do_GetIOService(&rv);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIProtocolHandler> protocolHandler;
@@ -276,16 +273,16 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
   nsCOMPtr<nsIPrincipal> loadingPrincipal =
       NullPrincipal::CreateWithoutOriginAttributes();
   if (MOZ_UNLIKELY(!(loadingPrincipal))) {
-    return (rv = NS_ERROR_NULL_POINTER);
+    return NS_ERROR_NULL_POINTER;
   }
 
-  nsCOMPtr<nsILoadInfo> loadInfo = new mozilla::net::LoadInfo(
+  nsCOMPtr<nsILoadInfo> loadInfo = MOZ_TRY(net::LoadInfo::Create(
       loadingPrincipal,
       nullptr,  // aTriggeringPrincipal
       nullptr,  // aLoadingNode
       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_INHERITS_SEC_CONTEXT |
           nsILoadInfo::SEC_ALLOW_CHROME | nsILoadInfo::SEC_DISALLOW_SCRIPT,
-      nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON);
+      nsIContentPolicy::TYPE_INTERNAL_IMAGE_FAVICON));
 
   nsCOMPtr<nsIChannel> channel;
   rv = protocolHandler->NewChannel(aDataURL, loadInfo, getter_AddRefs(channel));
@@ -300,7 +297,7 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
   rv = stream->Available(&available64);
   NS_ENSURE_SUCCESS(rv, rv);
   if (available64 == 0 || available64 > UINT32_MAX / sizeof(uint8_t)) {
-    return (rv = NS_ERROR_FILE_TOO_BIG);
+    return NS_ERROR_FILE_TOO_BIG;
   }
   uint32_t available = (uint32_t)available64;
 
@@ -311,7 +308,7 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
   rv = stream->Read(TO_CHARBUFFER(buffer.Elements()), available, &numRead);
   NS_ENSURE_SUCCESS(rv, rv);
   if (numRead != available) {
-    return (rv = NS_ERROR_UNEXPECTED);
+    return NS_ERROR_UNEXPECTED;
   }
 
   nsAutoCString mimeType;
@@ -319,7 +316,7 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
   NS_ENSURE_SUCCESS(rv, rv);
   if (!imgLoader::SupportImageWithMimeType(
           mimeType, AcceptedMimeTypes::IMAGES_AND_DOCUMENTS)) {
-    return (rv = NS_ERROR_UNEXPECTED);
+    return NS_ERROR_UNEXPECTED;
   }
 
   // If URI is a Data URI, mime type returned by channel may be incorrect, since
@@ -407,10 +404,13 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
       new AsyncSetIconForPage(icon, page, promise);
   RefPtr<Database> DB = Database::GetDatabase();
   if (MOZ_UNLIKELY(!DB)) {
-    return (rv = NS_ERROR_UNEXPECTED);
+    return NS_ERROR_UNEXPECTED;
   }
 
-  DB->DispatchToAsyncThread(event);
+  rv = DB->DispatchToAsyncThread(event);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
   guard.release();
   promise.forget(aPromise);
@@ -418,12 +418,44 @@ nsFaviconService::SetFaviconForPage(nsIURI* aPageURI, nsIURI* aFaviconURI,
 }
 
 NS_IMETHODIMP
-nsFaviconService::GetFaviconURLForPage(nsIURI* aPageURI,
-                                       nsIFaviconDataCallback* aCallback,
-                                       uint16_t aPreferredWidth) {
+nsFaviconService::GetFaviconForPage(nsIURI* aPageURI, uint16_t aPreferredWidth,
+                                    JSContext* aContext = nullptr,
+                                    dom::Promise** _retval = nullptr) {
   MOZ_ASSERT(NS_IsMainThread());
   NS_ENSURE_ARG(aPageURI);
-  NS_ENSURE_ARG(aCallback);
+
+  ErrorResult errorResult;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aContext), errorResult);
+  if (NS_WARN_IF(errorResult.Failed())) {
+    return errorResult.StealNSResult();
+  }
+
+  RefPtr<FaviconPromise> result =
+      AsyncGetFaviconForPage(aPageURI, aPreferredWidth);
+  result->Then(GetMainThreadSerialEventTarget(), __func__,
+               [promise](const FaviconPromise::ResolveOrRejectValue& aValue) {
+                 if (aValue.IsResolve()) {
+                   nsCOMPtr<nsIFavicon> favicon = aValue.ResolveValue();
+                   if (favicon) {
+                     promise->MaybeResolve(favicon);
+                   } else {
+                     promise->MaybeResolve(JS::NullHandleValue);
+                   }
+                 } else {
+                   promise->MaybeReject(aValue.RejectValue());
+                 }
+               });
+
+  promise.forget(_retval);
+  return NS_OK;
+}
+
+RefPtr<FaviconPromise> nsFaviconService::AsyncGetFaviconForPage(
+    nsIURI* aPageURI, uint16_t aPreferredWidth, bool aOnConcurrentConn) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(aPageURI);
+
   // Use the default value, may be UINT16_MAX if a default is not set.
   if (aPreferredWidth == 0) {
     aPreferredWidth = mDefaultIconURIPreferredSize;
@@ -431,82 +463,117 @@ nsFaviconService::GetFaviconURLForPage(nsIURI* aPageURI,
 
   nsCOMPtr<nsIURI> pageURI = GetExposableURI(aPageURI);
 
-  RefPtr<AsyncGetFaviconURLForPage> event =
-      new AsyncGetFaviconURLForPage(pageURI, aPreferredWidth, aCallback);
+  RefPtr<FaviconPromise::Private> promise =
+      new FaviconPromise::Private(__func__);
 
-  RefPtr<Database> DB = Database::GetDatabase();
-  NS_ENSURE_STATE(DB);
-  DB->DispatchToAsyncThread(event);
+  RefPtr<AsyncGetFaviconForPageRunnable> runnable =
+      new AsyncGetFaviconForPageRunnable(pageURI, aPreferredWidth, promise,
+                                         aOnConcurrentConn);
 
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsFaviconService::GetFaviconDataForPage(nsIURI* aPageURI,
-                                        nsIFaviconDataCallback* aCallback,
-                                        uint16_t aPreferredWidth) {
-  MOZ_ASSERT(NS_IsMainThread());
-  NS_ENSURE_ARG(aPageURI);
-  NS_ENSURE_ARG(aCallback);
-  // Use the default value, may be UINT16_MAX if a default is not set.
-  if (aPreferredWidth == 0) {
-    aPreferredWidth = mDefaultIconURIPreferredSize;
+  if (!aOnConcurrentConn) {
+    RefPtr<Database> DB = Database::GetDatabase();
+    if (MOZ_UNLIKELY(!DB)) {
+      promise->Reject(NS_ERROR_UNEXPECTED, __func__);
+    } else {
+      DB->DispatchToAsyncThread(runnable);
+    }
+  } else {
+    auto conn = ConcurrentConnection::GetInstance();
+    if (MOZ_UNLIKELY(!conn.isSome())) {
+      promise->Reject(NS_ERROR_UNEXPECTED, __func__);
+    } else {
+      conn.value()->Queue(runnable);
+    }
   }
 
-  nsCOMPtr<nsIURI> pageURI = GetExposableURI(aPageURI);
-
-  RefPtr<AsyncGetFaviconDataForPage> event =
-      new AsyncGetFaviconDataForPage(pageURI, aPreferredWidth, aCallback);
-  RefPtr<Database> DB = Database::GetDatabase();
-  NS_ENSURE_STATE(DB);
-  DB->DispatchToAsyncThread(event);
-
-  return NS_OK;
+  return promise;
 }
 
 NS_IMETHODIMP
-nsFaviconService::CopyFavicons(nsIURI* aFromPageURI, nsIURI* aToPageURI,
-                               uint32_t aFaviconLoadType,
-                               nsIFaviconDataCallback* aCallback) {
+nsFaviconService::TryCopyFavicons(nsIURI* aFromPageURI, nsIURI* aToPageURI,
+                                  uint32_t aFaviconLoadType,
+                                  JSContext* aContext = nullptr,
+                                  mozilla::dom::Promise** _retval = nullptr) {
   MOZ_ASSERT(NS_IsMainThread());
-  NS_ENSURE_ARG(aFromPageURI);
-  NS_ENSURE_ARG(aToPageURI);
-  NS_ENSURE_TRUE(
-      aFaviconLoadType >= nsIFaviconService::FAVICON_LOAD_PRIVATE &&
-          aFaviconLoadType <= nsIFaviconService::FAVICON_LOAD_NON_PRIVATE,
-      NS_ERROR_INVALID_ARG);
+
+  ErrorResult errorResult;
+  RefPtr<mozilla::dom::Promise> promise = mozilla::dom::Promise::Create(
+      xpc::CurrentNativeGlobal(aContext), errorResult);
+  if (NS_WARN_IF(errorResult.Failed())) {
+    return errorResult.StealNSResult();
+  }
+
+  RefPtr<mozilla::places::BoolPromise> result =
+      AsyncTryCopyFavicons(aFromPageURI, aToPageURI, aFaviconLoadType);
+  result->Then(
+      GetMainThreadSerialEventTarget(), __func__,
+      [promise](
+          const mozilla::places::BoolPromise::ResolveOrRejectValue& aValue) {
+        if (aValue.IsResolve()) {
+          promise->MaybeResolve(aValue.ResolveValue());
+        } else {
+          promise->MaybeReject(aValue.RejectValue());
+        }
+      });
+
+  promise.forget(_retval);
+  return NS_OK;
+}
+
+RefPtr<mozilla::places::BoolPromise> nsFaviconService::AsyncTryCopyFavicons(
+    nsCOMPtr<nsIURI> aFromPageURI, nsCOMPtr<nsIURI> aToPageURI,
+    uint32_t aFaviconLoadType) {
+  MOZ_ASSERT(NS_IsMainThread());
+
+  RefPtr<mozilla::places::BoolPromise::Private> promise =
+      new mozilla::places::BoolPromise::Private(__func__);
+
+  if (MOZ_UNLIKELY(!aFromPageURI)) {
+    promise->Reject(NS_ERROR_INVALID_ARG, __func__);
+    return promise;
+  }
+  if (MOZ_UNLIKELY(!aToPageURI)) {
+    promise->Reject(NS_ERROR_INVALID_ARG, __func__);
+    return promise;
+  }
+  if (MOZ_UNLIKELY(!canStoreIconForPage(aToPageURI))) {
+    promise->Reject(NS_ERROR_INVALID_ARG, __func__);
+    return promise;
+  }
+  if (!(aFaviconLoadType >= nsIFaviconService::FAVICON_LOAD_PRIVATE &&
+        aFaviconLoadType <= nsIFaviconService::FAVICON_LOAD_NON_PRIVATE)) {
+    promise->Reject(NS_ERROR_INVALID_ARG, __func__);
+    return promise;
+  }
 
   nsCOMPtr<nsIURI> fromPageURI = GetExposableURI(aFromPageURI);
   nsCOMPtr<nsIURI> toPageURI = GetExposableURI(aToPageURI);
 
-  PageData fromPage;
-  nsresult rv = fromPageURI->GetSpec(fromPage.spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-  PageData toPage;
-  rv = toPageURI->GetSpec(toPage.spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  NS_ENSURE_ARG(canStoreIconForPage(aToPageURI));
-
   bool canAddToHistory;
   nsNavHistory* navHistory = nsNavHistory::GetHistoryService();
-  NS_ENSURE_TRUE(navHistory, NS_ERROR_OUT_OF_MEMORY);
-  rv = navHistory->CanAddURI(toPageURI, &canAddToHistory);
-  NS_ENSURE_SUCCESS(rv, rv);
-  toPage.canAddToHistory =
-      !!canAddToHistory &&
-      aFaviconLoadType != nsIFaviconService::FAVICON_LOAD_PRIVATE;
+  if (MOZ_UNLIKELY(!navHistory)) {
+    promise->Reject(NS_ERROR_OUT_OF_MEMORY, __func__);
+    return promise;
+  }
+  nsresult rv = navHistory->CanAddURI(toPageURI, &canAddToHistory);
+  if (NS_FAILED(rv)) {
+    promise->Reject(rv, __func__);
+    return promise;
+  }
+  canAddToHistory = !!canAddToHistory &&
+                    aFaviconLoadType != nsIFaviconService::FAVICON_LOAD_PRIVATE;
 
-  RefPtr<AsyncCopyFavicons> event =
-      new AsyncCopyFavicons(fromPage, toPage, aCallback);
-
-  // Get the target thread and start the work.
-  // DB will be updated and observers notified when done.
+  RefPtr<AsyncTryCopyFaviconsRunnable> runnable =
+      new AsyncTryCopyFaviconsRunnable(fromPageURI, toPageURI, canAddToHistory,
+                                       promise);
   RefPtr<Database> DB = Database::GetDatabase();
-  NS_ENSURE_STATE(DB);
-  DB->DispatchToAsyncThread(event);
+  if (MOZ_UNLIKELY(!DB)) {
+    promise->Reject(NS_ERROR_UNEXPECTED, __func__);
+    return promise;
+  }
+  DB->DispatchToAsyncThread(runnable);
 
-  return NS_OK;
+  return promise;
 }
 
 nsresult nsFaviconService::GetFaviconLinkForIcon(nsIURI* aFaviconURI,
@@ -665,29 +732,6 @@ nsresult nsFaviconService::OptimizeIconSizes(IconData& aIcon) {
   return aIcon.payloads.IsEmpty() ? NS_ERROR_FILE_TOO_BIG : NS_OK;
 }
 
-nsresult nsFaviconService::GetFaviconDataAsync(
-    const nsCString& aFaviconSpec, mozIStorageStatementCallback* aCallback) {
-  MOZ_ASSERT(aCallback, "Doesn't make sense to call this without a callback");
-
-  nsCOMPtr<nsIURI> uri;
-  nsresult rv = NS_NewURI(getter_AddRefs(uri), aFaviconSpec);
-  NS_ENSURE_SUCCESS(rv, rv);
-  uri = GetExposableURI(uri);
-
-  nsCOMPtr<mozIStorageAsyncStatement> stmt = mDB->GetAsyncStatement(
-      "/*Do not warn (bug no: not worth adding an index */ "
-      "SELECT data, width FROM moz_icons "
-      "WHERE fixed_icon_url_hash = hash(fixup_url(:url)) AND icon_url = :url "
-      "ORDER BY width DESC");
-  NS_ENSURE_STATE(stmt);
-
-  rv = URIBinder::Bind(stmt, "url"_ns, uri);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<mozIStoragePendingStatement> pendingStatement;
-  return stmt->ExecuteAsync(aCallback, getter_AddRefs(pendingStatement));
-}
-
 NS_IMETHODIMP
 nsFaviconService::SetDefaultIconURIPreferredSize(uint16_t aDefaultSize) {
   mDefaultIconURIPreferredSize = aDefaultSize > 0 ? aDefaultSize : UINT16_MAX;
