@@ -43,13 +43,15 @@
 #ifdef MOZ_BACKGROUNDTASKS
 #  include "mozilla/BackgroundTasks.h"
 #endif
+#include "mozilla/CmdLineAndEnvUtils.h"
 #include "mozilla/Components.h"
 #include "mozilla/Services.h"
 #include "mozilla/Omnijar.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/ProfilerLabels.h"
-#include "mozilla/Telemetry.h"
+#include "mozilla/glean/ToolkitXreMetrics.h"
 #include "mozilla/Try.h"
+#include "mozilla/Utf8.h"
 #include "mozilla/XREAppData.h"
 #include "nsPrintfCString.h"
 
@@ -94,8 +96,8 @@
 nsXREDirProvider* gDirServiceProvider = nullptr;
 nsIFile* gDataDirHomeLocal = nullptr;
 nsIFile* gDataDirHome = nullptr;
-MOZ_RUNINIT nsCOMPtr<nsIFile> gDataDirProfileLocal = nullptr;
-MOZ_RUNINIT nsCOMPtr<nsIFile> gDataDirProfile = nullptr;
+MOZ_CONSTINIT nsCOMPtr<nsIFile> gDataDirProfileLocal{};
+MOZ_CONSTINIT nsCOMPtr<nsIFile> gDataDirProfile{};
 
 // These are required to allow nsXREDirProvider to be usable in xpcshell tests.
 // where gAppData is null.
@@ -621,7 +623,7 @@ nsXREDirProvider::DoStartup() {
       if (db) {
         nsCOMPtr<nsIPK11Token> token;
         if (NS_SUCCEEDED(db->GetInternalKeyToken(getter_AddRefs(token)))) {
-          mozilla::Unused << token->Login(false);
+          (void)token->Login(false);
         }
       } else {
         NS_WARNING("Failed to get nsIPK11TokenDB service.");
@@ -667,7 +669,7 @@ nsXREDirProvider::DoStartup() {
       else
         mode = 2;
     }
-    mozilla::Telemetry::Accumulate(mozilla::Telemetry::SAFE_MODE_USAGE, mode);
+    mozilla::glean::gecko::safe_mode_usage.AccumulateSingleSample(mode);
 
     obsSvc->NotifyObservers(nullptr, "profile-initial-state", nullptr);
   }
@@ -1153,6 +1155,9 @@ nsresult nsXREDirProvider::GetSystemExtensionsDirectory(nsIFile** aFile) {
 
 nsresult nsXREDirProvider::GetUserDataDirectory(nsIFile** aFile, bool aLocal) {
   nsCOMPtr<nsIFile> localDir;
+  nsCOMPtr<nsIFile> customDir = mozilla::GetFileFromEnv("MOZ_APP_DATA");
+  nsCOMPtr<nsIFile> customLocalDir =
+      mozilla::GetFileFromEnv("MOZ_LOCAL_APP_DATA");
 
   if (aLocal && gDataDirProfileLocal) {
     return gDataDirProfileLocal->Clone(aFile);
@@ -1161,11 +1166,18 @@ nsresult nsXREDirProvider::GetUserDataDirectory(nsIFile** aFile, bool aLocal) {
     return gDataDirProfile->Clone(aFile);
   }
 
-  nsresult rv = GetUserDataDirectoryHome(getter_AddRefs(localDir), aLocal);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv;
+  if (aLocal && customLocalDir) {
+    localDir = customLocalDir;
+  } else if (!aLocal && customDir) {
+    localDir = customDir;
+  } else {
+    rv = GetUserDataDirectoryHome(getter_AddRefs(localDir), aLocal);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = AppendProfilePath(localDir, aLocal);
-  NS_ENSURE_SUCCESS(rv, rv);
+    rv = AppendProfilePath(localDir, aLocal);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   rv = EnsureDirectoryExists(localDir);
   NS_ENSURE_SUCCESS(rv, rv);
